#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (c) 2008-2013 Nestor Arocha Rodriguez

"""LL(1) Recursive Descent Lexer
second recipe of the book "Language implementation patterns

grammar NestedNameList;
list : '[' elements ']' ; // match bracketed list
elements : element (',' element)* ; // match comma-separated list
element : NAME | list ; // element is name or nested list
NAME : ('a'..'z' |'A'..'Z' )+ ; // NAME is sequence of >=1 letter
"""

#tokenlist = ["NAME", "COMMA", "LBRACK", "RBRACK","EOF_TYPE"]

from pydsl.Alphabet.Lexer import Lexer as _Lexer

class _Parser:
    def __init__(self, lexer, size = 3):
        self.lexer = lexer
        self.currenttoken = None
        self.lookahead = []
        self.size = size
        for _ in range(size):
            self.consume()

    def match(self, x):
        if self.lookahead[0][0] == x:
            self.consume()
        else:
            raise Exception

    def consume(self):
        self.lookahead.append(self.lexer.nextToken())
        if len(self.lookahead) > 3:
            self.lookahead.pop(0)

class _ListParser(_Parser):
    def rlist(self):
        self.match("LBRACK")
        self.elements()
        self.match("RBRACK")

    def elements(self):
        self.element()
        while self.lookahead[0][0] == "COMMA":
            self.match("COMMA")
            self.element()

    def element(self):
        if self.lookahead[0][0] == "NAME" and self.lookahead[1][0] == "EQUAL":
            self.match("NAME")
            self.match("EQUAL")
            self.match("NAME")
        elif self.lookahead[0][0] == "NAME":
            self.match("NAME")
        elif self.lookahead[0][0] == "LBRACK":
            self.rlist()
        else: 
            raise Exception



class _ListLexer(_Lexer):
    def nextToken(self):
        import re
        from pydsl.Alphabet.Lexer import finalchar
        while self.current != finalchar:
            if self.current == "/":
                self.comment(tl)
                continue
            elif self.current == " ":
                self.consume()
                continue
            elif self.current == ",":
                return self.comma()
            elif self.current == "[":
                return self.lbrack()
            elif self.current == "]":
                return self.rbrack()
            elif self.current == "=":
                return self.equals()
            elif re.match("[a-zA-Z]", self.current):
                return self.name()
            else:
                raise Exception
        return "EOF_TYPE", ""

    def comma(self):
        current = self.current
        self.match(",")
        return "COMMA", current

    def equals(self):
        current = self.current
        self.match("=")
        return "EQUAL", current

    def lbrack(self):
        current = self.current
        self.match("[")
        return "LBRACK", current

    def rbrack(self):
        current = self.current
        self.match("]")
        return "RBRACK", current

    def name(self):
        import re
        string = ""
        from pydsl.Grammar.Lexer import finalchar
        while self.current != finalchar and re.match("[a-zA-Z]", self.current):
            string += self.current
            self.consume()
        return "NAME", string


def function(inputdic, inputgrammar, outputdic):
    a = _ListLexer(inputdic["input"])
    b = _ListParser(a)
    result = b.rlist()

    return {"output":result}


iclass = "PythonTransformer"
inputdic = {"input":"cstring"}
outputdic = {"output":"cstring"}





#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.


from pydsl.Query import QueryEquality, QueryInclusion, QueryGreaterThan, QueryPartial
import collections

class Indexer(object):
    """Indexes memory content. Current implementation just copy Memory iterator (Inmutabledicts) into a cache dict. This is possible because Memory iterator format is suitable for search, but it might change in the future"""
    def __init__(self, memory):
        self.memory = memory #One memory per indexer
        self.index = []
        self.__init_index()

    def __init_index(self):
        """Stores all data in the index"""
        for element in self.memory:
            if not isinstance(element, collections.Hashable):
                raise TypeError("Adding non hashable element %s to index" % str(element))
            self.index.append(element)

    def show_all(self):# -> list:
        """All elements inside index"""
        return list(self.index)

    def __update_index(self):
        """Updates index content"""
        for element in self.memory:
            if element not in self.index:
                self.index.append(element)

    def search_index(self, queryterm): #-> set:
        result = set()
        qpart = queryterm.right
        if isinstance(queryterm, QueryEquality):
            for element in self.index:
                if not element:
                    continue
                elementright = None
                if queryterm.left.count('.') > 0:
                    getlist = queryterm.left.split('.')
                    try:
                        elementright = element[getlist[0]]
                        for getindex in getlist[1:]:
                            elementright = elementright.__getitem__(getindex)
                    except KeyError:
                        continue
                if not elementright and not queryterm.left in element:
                    continue
                if not elementright:
                    elementright = element[queryterm.left]
                ismatch = True
                try:
                    if len(qpart)>2 and qpart[-1] == "/" and qpart[0] == "/":
                        #Expresion regular
                        import re
                        rexp = re.compile(qpart[1:-1])
                        if rexp.match(elementright) is None:
                            ismatch = False
                    else:
                        #Cadena normal
                        if qpart != elementright:
                            ismatch = False
                except KeyError:
                    continue
                if ismatch:
                    if isinstance(element, dict):
                        from pydsl.Abstract import InmutableDict
                        result.add(InmutableDict(element))
                    else:
                        result.add(element)
        elif isinstance(queryterm, QueryInclusion):
            qpart = queryterm.right
            for element in self.index:
                ismatch = True
                try:
                    if qpart[-1] == "/" and qpart[0] == "/":
                        #RegExp
                        import re
                        rexp = re.compile(qpart[1:-1])
                        ismatch = False
                        for item in element[queryterm.left]:
                            if rexp.match(item) is not None:
                                ismatch = True
                                break
                    else:
                        #string
                        if qpart not in element[queryterm.left]:
                            ismatch = False
                            continue
                except KeyError:
                    continue
                if ismatch:
                    if isinstance(element, dict):
                        from pydsl.Abstract import InmutableDict
                        result.add(InmutableDict(element))
                    else:
                        result.add(element)
        elif isinstance(queryterm, QueryPartial):
            rdict = queryterm.right
            for element in self.index:
                ismatch = True
                if queryterm.left not in element:
                    continue
                for key, value in rdict.items():
                    try:
                        if key not in element[queryterm.left]:
                            ismatch = False
                            break
                        if element[queryterm.left][key] != value:
                            ismatch = False
                    except KeyError:
                        continue
                if ismatch:
                    if isinstance(element, dict):
                        from pydsl.Abstract import InmutableDict
                        result.add(InmutableDict(element))
                    else:
                        result.add(element)
        elif isinstance(queryterm, QueryGreaterThan):
            try:
                qpart = int(queryterm.right)
            except ValueError:
                return set()
            for element in self.index:
                #TODO: "." operator for dict member access
                ismatch = False
                try:
                    #Cadena normal
                    if qpart == int(element[queryterm.left]):
                        ismatch = True
                        break
                except KeyError:
                    continue
                except ValueError:
                    continue
                if ismatch:
                    if isinstance(element, dict):
                        from pydsl.Abstract import InmutableDict
                        result.add(InmutableDict(element))
                    else:
                        result.add(element)
        return result





#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (c) 2008-2013 Nestor Arocha Rodriguez

"""Memoizing parser

grammar NestedNameList;
stat: list EOF | assing EOF;
assing : list '=' list
list : '[' elements ']' ; // match bracketed list
elements : element (',' element)* ; // match comma-separated list
element : NAME '=' NAME | NAME | list ; // element is name or nested list
NAME : ('a'..'z' |'A'..'Z' )+ ; // NAME is sequence of >=1 letter
"""

#tokenlist = ["NAME", "COMMA", "LBRACK", "RBRACK","EOF_TYPE"]



from pydsl.Alphabet.Lexer import Lexer as _Lexer


class memoized:
    def __init__(self, name):
        self.name = name 

    def __call__(self, func):
        def wrap(instance):
            failed = False
            if instance.speculating() and instance.alreadyParserRule(self.name, instance.index):
                return
            previndex = instance.index
            try:
                func(instance)
            except Exception as e:
                failed = True
            finally:
                if instance.speculating():
                    result = -1
                    if not failed:
                        result = instance.index
                    instance.memoize[(self.name, previndex)] = result
        return wrap              


class _Parser:
    def __init__(self, lexer):
        self.lexer = lexer
        self.lookahead = []
        self.markers = []
        self.index = 0
        self.memoize = {}

    def alreadyParserRule(self, name, index):
        memo = self.memoize.get((name, index))
        if not memo:
            return False
        if memo== -1:
            raise Exception
        self.index = memo
        return True

    def LT(self, i):
        """gets element from input"""
        self.sync(i)
        return self.lookahead[self.index+i-1]

    def match(self, x):
        if self.LT(1)[0] == x:
            self.consume()
        else:
            raise Exception("Not matched")

    def sync(self, i):
        """Fills from index to index +i """
        if self.index + i > len(self.lookahead):
            n = self.index + i - len(self.lookahead)
            self.fill(n)


    def fill(self, n):
        """fills n elements"""
        for _ in range(n):
            self.lookahead.append(self.lexer.nextToken())

    def consume(self):
        self.index += 1
        if self.index == len(self.lookahead) and not self.speculating():
            self.index = 0
            self.lookahead = []
            self.memoize = {}
        self.sync(1)

    def mark(self):
        self.markers.append(self.index)
        return self.index

    def release(self):
        marker = self.markers.pop()
        self.index = marker

    def speculating(self):
        return len(self.markers) > 0

class _ListParser(_Parser):
    def stat(self):
        if self.speculate_stat_1():
            self.rlist()
            self.match("EOF_TYPE")
        elif self.speculate_stat_2():
            self.assign()
            self.match("EOF_TYPE")
        else:
            raise Exception("No alternative for stat")

    def speculate_stat_1(self):
        success = True
        self.mark()
        try:
            self.rlist()
            self.match("EOF_TYPE")
        except Exception as e:
            success = False
        self.release()
        return success

    def speculate_stat_2(self):
        success = True
        self.mark()
        try:
            self.assign()
            self.match("EOF_TYPE")
        except Exception as e:
            success = False
        self.release()
        return success

    def assign(self):
        self.rlist()
        self.match("EQUAL")
        self.rlist()


    @memoized("rlist")
    def rlist(self):
        self.match("LBRACK")
        self.elements()
        self.match("RBRACK")

    def elements(self):
        self.element()
        while self.LT(1)[0] == "COMMA":
            self.match("COMMA")
            self.element()

    def element(self):
        if self.LT(1)[0] == "NAME" and self.LT(2)[0] == "EQUAL":
            self.match("NAME")
            self.match("EQUAL")
            self.match("NAME")
        elif self.LT(1)[0] == "NAME":
            self.match("NAME")
        elif self.LT(1)[0] == "LBRACK":
            self.rlist()
        else: 
            raise Exception



class _ListLexer(_Lexer):
    def nextToken(self):
        import re
        from pydsl.Grammar.Lexer import finalchar
        while self.current != finalchar:
            if self.current == "/":
                self.comment(tl)
                continue
            elif self.current == " ":
                self.consume()
                continue
            elif self.current == ",":
                return self.comma()
            elif self.current == "[":
                return self.lbrack()
            elif self.current == "]":
                return self.rbrack()
            elif self.current == "=":
                return self.equals()
            elif re.match("[a-zA-Z]", self.current):
                return self.name()
            else:
                raise Exception
        return "EOF_TYPE", ""

    def comma(self):
        current = self.current
        self.match(",")
        return "COMMA", current

    def equals(self):
        current = self.current
        self.match("=")
        return "EQUAL", current

    def lbrack(self):
        current = self.current
        self.match("[")
        return "LBRACK", current

    def rbrack(self):
        current = self.current
        self.match("]")
        return "RBRACK", current

    def name(self):
        import re
        string = ""
        from pydsl.Grammar.Lexer import finalchar
        while self.current != finalchar and re.match("[a-zA-Z]", self.current):
            string += self.current
            self.consume()
        return "NAME", string


def function(inputdic, inputgrammar, outputdic):
    a = _ListLexer(inputdic["input"])
    b = _ListParser(a)
    result = b.stat()

    return {"output":result}


iclass = "PythonTransformer"
inputdic = {"input":"cstring"}
outputdic = {"output":"cstring"}





#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""
Searcher class
"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

#En esta implementacion, el searcher se enlaza con un indexer. El usuario introduce un Query, y se utiliza el indexer para averiguar que documentos cumplen cada una de las condiciones. El uso de operadores and y or se realizara despues de obtener los resultados con operaciones de conjuntos

#A searcher is linked with an indexer. Users creates a query, and then searcher iterates the indexer to find matches

from .Indexer import Indexer

class Searcher(object):
    def search(self, query):
        raise NotImplementedError

class MemorySearcher(Searcher):
    def __init__(self, indexerlist):
        from pydsl.Memory.Memory import Memory
        if isinstance(indexerlist, Indexer):
            indexerlist = [indexerlist]
        elif isinstance(indexerlist, Memory):
            indexerlist = [Indexer(indexerlist)]
        assert(isinstance(indexerlist, list))
        self.indexerlist = indexerlist

    def search(self, query = None): # -> set:
        if isinstance(query, str):
            from .Query import str_to_memoryquery
            query = str_to_memoryquery(query)
        elif isinstance(query, dict):
            from .Query import dict_to_query
            query = dict_to_query(query)
        if not query:
            result = set()
            for indexer in self.indexerlist:
                result = result.union(indexer.show_all())
            return result
        return self.__recursive_search(query.content)

    def __recursive_search(self, queryelement): # -> set:
        from pydsl.Query import NotQueryOperator, AndQueryOperator, OrQueryOperator, QueryTerm
        if isinstance(queryelement, AndQueryOperator):
            r1 = self.__recursive_search(queryelement.element1)
            r2 = self.__recursive_search(queryelement.element2)
            return r1.intersection(r2)
        elif isinstance(queryelement, OrQueryOperator):
            r1 = self.__recursive_search(queryelement.element1)
            r2 = self.__recursive_search(queryelement.element2)
            return r1.union(r2)
        elif isinstance(queryelement, NotQueryOperator):
            total = set()
            for indexer in self.indexerlist:
                total = total.union(indexer.show_all())
            s1 = self.__recursive_search(queryelement.element)
            return total.difference(s1)
        elif isinstance(queryelement, QueryTerm):
            result = set()
            for indexer in self.indexerlist:
                result = result.union(indexer.search_index(queryelement))
            return result
        else:
            raise TypeError





#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"Tree class for tree based parsers"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
LOG = logging.getLogger(__name__)
from pydsl.Grammar.Symbol import TerminalSymbol

def traversePreOrder(item):
    result = [item]
    for child in item.childlist:
        result += traversePreOrder(child)
    return result

def traverseInOrder(item):
    result = [traverseInOrder(item.childlist[0]), item]
    for child in item.childlist[1:]:
        result += traverseInOrder(child)
    return result

def traversePostOrder(item):
    result = []
    for child in item.childlist:
        result += traversePostOrder(child)
    result.append(item)
    return result

class Tree(object):
    def __init__(self, childlist=None):
        if not childlist: childlist = []
        self.childlist = childlist

    def append_child(self, dpr):
        """appends dpr to childlist"""
        self.childlist.append(dpr)

    def to_list(self, order = "preorder"):
        if order == "preorder":
            return traversePreOrder(self)
        elif order == "inorder":
            return traverseInOrder(self)
        elif order == "postorder":
            return traversePostOrder(self)
        else:
            raise KeyError

    def first_leaf(self):
        """Returns the first lead node"""
        if self.childlist:
            return self.childlist[0].first_leaf()
        else:
            return self

class PositionTree(Tree):
    """Stores the position of the original tree"""
    def __init__(self, leftpos, rightpos, content, production = None, valid = True, childlist = []):
        self.leftpos = leftpos
        self.rightpos = rightpos
        self.childlist = list(childlist) #creates a copy
        self.content = content
        self.valid = valid
        self.production = production
        
    def __eq__(self, other):
        try:
            return self.production == other.production and self.content == other.content and self.leftpos == other.leftpos and self.rightpos == other.rightpos and self.valid == other.valid
        except AttributeError:
            return False

    def __bool__(self):
        """checks if it is a null result"""
        return self.valid

    def __getitem__(self, key, order = "preorder"): #FIXME: getitem and optional argument??
        result = []
        mylist = self.to_list(order)
        for element in mylist:
            if element.content == key:
                result.append(element)
        if not result:
            raise KeyError
        return result

    def __str__(self):
        result = "<PositionTree: "
        result += "(" + str(self.leftpos) + "," + str(self.rightpos)
        if self.childlist:
            result += ", children: " + str(self.childlist)
        result += " >"
        return result

    def shift(self, amount):
        """ shifts position """
        if self.leftpos is not None:
            self.leftpos += amount
        if self.leftpos is not None:
            self.rightpos += amount

    def __len__(self):
        if self.rightpos is None and self.leftpos is None:
            return 0
        return self.rightpos - self.leftpos

    def coverage(self):
        if not self:
            return 0, len(self)
        if self.childlist:
            childtotal = 0
            childcoverage = 0
            for child in self.childlist:
                newcoverage, newtotal = child.coverage()
                childcoverage += newcoverage
                childtotal += newtotal
            assert(childtotal == len(self))
            return childcoverage, childtotal
        else:
            return len(self), len(self)

    def get_by_symbol(self, index):
        if isinstance(self.production, TerminalSymbol):
            return [(self.leftpos, self.rightpos)] # FIXME quick hack for terminal rules
        result = []
        if self.production.leftside[0].name == index:
            result.append((self.leftpos, self.rightpos))
        else:
            LOG.debug("Not equal: " + str(self.production.leftside[0].name) + " and :" + str(index))
        for child in self.childlist:
            result += child.get_by_symbol(index)
        return result

    def __contains__(self, index):
        if isinstance(self.production, TerminalSymbol):
            return index == self.production.name # FIXME quick hack for terminal rules
        if not self.production.leftside:
            return False
        if self.production.leftside[0].name == index:
            return True
        for child in self.childlist:
            if child.get_by_symbol(index):
                return True
        return False


class ParseTree(PositionTree):
    """ Stores a descent parser iteration result """
    def __init__(self, leftpos, rightpos, symbollist, content, production, childlist = [], valid = True):
        if not isinstance(leftpos, int) and leftpos is not None:
            raise TypeError
        if not isinstance(rightpos, int) and rightpos is not None:
            raise TypeError
        if not isinstance(symbollist, list):
            raise TypeError
        from .BNF import Production
        if production is not None and not (isinstance(production, Production) or
                isinstance(production, TerminalSymbol)):
            raise TypeError(production)
        PositionTree.__init__(self, leftpos, rightpos, content, production, valid, childlist)
        self.symbollist = symbollist

    def __add__(self, other):
        """ Adds two results. Only if self.rightpos = other.leftpos and parents are the same """
        if not isinstance(other, ParseTree):
            raise TypeError
        if other == []:
            return ParseTree(self.leftpos, self.rightpos, self.symbollist,
                    self.content, self.production, self.childlist) #FIXME: Must return a childlist copy
        if self.rightpos == other.leftpos and self.production == other.production:
            leftpos = self.leftpos
            rightpos = other.rightpos
            production = self.production
            content = self.content + other.content
            symbollist = self.symbollist + other.symbollist
            childlist = self.childlist + other.childlist
            return ParseTree(leftpos, rightpos, symbollist, content, production, childlist)
        else:
            LOG.warning("Unable to add parser results")
            raise Exception

    def __str__(self):
        result = "<ParseTree: "
        result += "(" + str(self.leftpos) + "," + str(self.rightpos)
        result += ") SymbolList: "
        result += str(self.symbollist)
        if self.childlist:
            result += ", children: " + str(self.childlist)
        result += " >"
        return result
#Original implementation: https://github.com/irskep/zhang-shasha and https://github.com/timtadh/zhang-shasha

def zss_distance(tree1, tree2):
    treedists = {}
    o1 = tree1.to_list("postorder")
    o2 = tree2.to_list("postorder")
    LOG.debug([x.content for x in o1])
    LOG.debug([x.content for x in o2])
    l1 = [o1.index(x.first_leaf()) for x in o1]
    l2 = [o2.index(x.first_leaf()) for x in o2]

    def treedist(i, j):
        def nodedist(a, b):
            if a == b:
                return 0
            assert(a.content != b.content) #For testing, but wrong
            return 1
        if i in treedists and j in treedists[i]: 
            return treedists[i][j] #Cached!
        def s(i, j, v):
            if i not in treedists: 
                treedists[i] = {} 
            treedists[i][j] = v
        forestdists = {}
        def gfd(a, b): # get an item from the forest dists dict
            if (a,b) in forestdists:
                return forestdists[(a,b)]
            if a[0] >= a[1] and b[0] >= b[1]: # δ(θ, θ) = 0
                return 0
            if b[0] >= b[1]:
                return forestdists[(a,(0,0))]
            if a[0] >= a[1]:
                return forestdists[((0,0),b)]

        for x in range(l1[i], i+1):
            forestdists[(l1[i], x), (0,0)] = gfd((l1[i],x-1),(0,0)) + 1

        for y in range(l2[j], j+1):
            forestdists[(0,0),(l2[j], y)] = gfd((0,0),(l2[j],y-1)) + 1

        for x in range(l1[i], i+1):
            for y in range(l2[j], j+1):
                if l1[i] == l1[x] and l2[j] == l2[y]:
                    #Inside the same keyroot
                    forestdists[((l1[i],x),(l2[j],y))] = min(
                            (gfd((l1[i],x-1),(l2[j],y)) + 1),
                            (gfd((l1[i],x),(l2[j],y-1)) + 1),
                            (gfd((l1[i],x-1),(l2[j],y-1)) + nodedist(o1[x], o2[y])))
                    s(x,y, forestdists[((l1[i],x),(l2[j],y))])
                else:
                    forestdists[((l1[i],x),(l2[j],y))] = min(
                            (gfd((l1[i],x-1),(l2[j],y)) + 1),
                            (gfd((l1[i],x),(l2[j],y-1)) + 1),
                            (gfd((l1[i],l1[x]-1),(l2[j],l2[y]-1)) + treedist(x,y)))

        return treedists[i][j]

    keyroots1 = {}
    keyroots2 = {}
    for i in range(len(o1)):
        keyroots1[l1[i]] = i
    for j in range(len(o2)):
        keyroots2[l2[j]] = j
    keyroots1 = sorted(keyroots1.values())
    keyroots2 = sorted(keyroots2.values())
    for x in keyroots1:
        for y in keyroots2:
            result = treedist(x, y)
            #print(x,y,result)
    return result



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""Python Transformers"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

from .Channel import HostChannel
from pydsl.Exceptions import ProcessingError
import logging
LOG = logging.getLogger("PythonTransformer")


class PythonTransformer(HostChannel):
    """ Python function based transformer """
    def __init__(self, inputdic, outputdic, function):
        HostChannel.__init__(self, inputdic, outputdic)
        self._function = function

    def __call__(self, *args, **kwargs):
        if kwargs:
            for inputkey in self.inputchanneldic.keys():
                if inputkey not in kwargs:
                    raise KeyError("Key %s not found in inputdic" % inputkey)
            ibdic = kwargs
        elif len(args) == 1 and isinstance(args[0], dict):
            ibdic = args[0]
        elif len(args) == 1:
            ibdic = {list(self.inputchanneldic.keys())[0]:args[0]}
        else:
            raise ValueError
        for dickey in ibdic.keys():
            if not self.inputchanneldic[dickey].check(ibdic[dickey]):
                raise ValueError("Invalid value %s for input %s (%s)" % (ibdic[dickey], dickey, self))
        result = self._functionwrapper(ibdic)
        return result

    def _functionwrapper(self, wdict):
        """Wraps function call, to add parammeters if required"""
        LOG.debug("PythonTransformer._functionwrapper: begin")
        if hasattr(self, "_hostT"):
            result = self._function(wdict, self._hostT, self.inputchanneldic, self.outputchanneldic)
        else:
            result = self._function(wdict, self.inputchanneldic, self.outputchanneldic)
        if not result:
            raise ProcessingError("Transformer", self)
        for outputgrammarname in self.outputchanneldic.keys():
            LOG.debug("Verifying Grammar name: " + outputgrammarname)
            if not outputgrammarname in result:
                LOG.error("Error while verifying Grammar name:" + outputgrammarname)
                raise ProcessingError("Transformer")
        return result

    def __str__(self):
        return "<PythonTransformer: %s, %s" % (self.inputdefinition, self.outputdefinition)

    @property
    def summary(self):
        from pydsl.Abstract import InmutableDict
        inputdic = tuple(self.inputdefinition.values())
        outputdic = tuple(self.outputdefinition.values())
        result = {"iclass": "PythonTransformer", "input": inputdic, "output": outputdic}
        return InmutableDict(result)


class HostPythonTransformer(PythonTransformer):
    """Python Function Transformer which can call to other functions"""
    def __init__(self, inputdic, outputdic, auxdic, function):
        PythonTransformer.__init__(self, inputdic, outputdic, function)
        self._hostT = {}
        self._initHostT(auxdic)

    def _initHostT(self, namedic):
        """Inits auxiliary transformers """
        from pydsl.Memory.Loader import load
        for title, gttype in namedic.items():
            self._hostT[title] = load(gttype)
            LOG.debug("loaded " + str(title) + "auxT")



#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (C) 2008-2013 Nestor Arocha Rodríguez

def function(inputdic, inputgt, outputgt):
    inputData = inputdic["input"]
    if inputData:  
        integerlist = inputData.split("+")
        if len(integerlist) == 2:
            output1 = integerlist[0]
            output2 = integerlist[1]
        else:
            output1 = ""
            output2 = ""
        return {"int1":output1, "int2":output2}

inputdic = {"input":"integerop"}
outputdic = {"int1":"integer","int2":"integer"}
iclass = "PythonTransformer"




#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""Protocols"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
LOG = logging.getLogger(__name__)

def protocol_split(content):
    """Splits a protocol string"""
    try:
        protocol, path = content.split("://")
    except ValueError:
        return {"protocol":""}
    result =  {"protocol":protocol}
    if "?" in path:
        npath, identifier = path.split("?")
        result["path"] = npath
        result["identifier"] = identifier
    else:
        result["path"] = path
    return result




#!/usr/bin/python
# -*- coding: utf-8 -*-

#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
from .Tool import GrammarTools
LOG = logging.getLogger("Grammar.Symbol")

class SymbolGrammarTools(GrammarTools):
    """Ask sentences to other grammars. Works with tokens"""
    def __init__(self, bnf, parser = "auto"):
        LOG.debug("SymbolGrammarTools.__init__: Begin")
        GrammarTools.__init__(self, bnf)
        parser = bnf.options.get("parser",parser)
        from pydsl.Memory.Loader import load_parser
        self.__parser = load_parser(bnf, parser)

    @property
    def productionset(self):
        """Returns parser productionset"""
        return self.__parser.productionset

    def groups(self):
        return [x.name for x in self.productionset.getSymbols()]

    def get_groups(self, word, propertyname):# -> list:
        """ Allow to ask for a grammar property of a valid sentence. Example: (English)ask for verb in a phrase: askProperty("verb","desk is clean") Maybe there is a method to pass it through __init__ (like _matchFun) """
        treelist = self.__parser.get_trees(word)
        for tree in treelist:
            if propertyname in tree:
                return tree.get_by_symbol(propertyname)
            return []

    def get_trees(self, word, showErrors = False): # -> list:
        """Returns a decomposition tree. askProperty and checkWord should rely on this
        parser returns DescentParserResult, but this function should return SymbolTokenTreeNode.
        It should work also for errors
        """
        return self.__parser.get_trees(word, showErrors)
    
    def genealogy(self, information, index): # -> list:
        """Given a word(token) index, will tell all parent symbols  until root node"""
        raise NotImplementedError

    def tokenize(self, information):
        #gets tree, then iterates through leaf nodes. 
        #TODO think about extra parameters for tree depth
        raise NotImplementedError



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"


class Token(object):
    """ Stores a symbol and its associated input """
    def __init__(self, terminalsymbol, string):
        self.symbol = terminalsymbol
        self.string = string

    def __str__(self):
        return self.string

class TokenList(list):
    def __bool__(self):
        return len(self) > 1

    def __str__(self):
        return "".join([str(x[1]) for x in self[:-1]])




#!/usr/bin/python3
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.


"""
Generic validator for BNF Grammars. 
Displays the part of the tree that doesn't conform the grammar definition
"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging

def print_errors(postnode):
    result = ""
    if not postnode.valid:
        result += "Error at production: " + str(postnode.production) + "(" + str(postnode.leftpos) + "," + str(postnode.rightpos) + ")"
        for child in postnode.childlist:
            result += print_errors(child) + "\n"
    return result

def errors_to_list(postnode):
    result = []
    if not postnode.valid:
        result.append((postnode.leftpos, postnode.rightpos, str(postnode.production)))
        for child in postnode.childlist:
            result += errors_to_list(child)
    return result

def validate(sgrammar, expression = None, inputfile = None, outputformat = None, **kwargs):
    """Read input file contents, creates grammar and transform objects, create connections, 
    and afterwards reads required input/launch main loop"""
    from pydsl.Memory.Loader import load_parser
    parser = load_parser(sgrammar, "descent")
    if expression:
        resulttrees = parser.get_trees(expression, True)
    elif inputfile:
        with open(inputfile, "rb") as f:
            resulttrees = parser.get_trees(f.read(), True)
    else:
        raise Exception("No input method")
    jsonlist = []
    for index, posttree in enumerate(resulttrees):
        if outputformat == "str":
            print("Tree: " + str(index) + "\n")
        if outputformat == "str":
            if posttree.valid:
                print("Result OK")
            else:
                print("Errors:")
                print(print_errors(posttree))
        elif outputformat == "json":
            jsonlist.append(errors_to_list(posttree))
    if outputformat == "json":
        import json
        print(json.dumps(jsonlist))
    return True

if __name__ == "__main__":
    import argparse
    TUSAGE = "usage: %(prog)s [options] [filename]"
    PARSER = argparse.ArgumentParser(usage = TUSAGE)
    PARSER.add_argument("-d", "--debuglevel", action="store", type=int, dest="debuglevel", help="Sets debug level")
    PARSER.add_argument("-i", "--inputfile", action="store", dest="inputfile", help="input filename ")
    PARSER.add_argument("-e", "--expression", action="store", dest="expression", help="input expression")
    PARSER.add_argument('-o', dest='outputformat',nargs='?', choices=["str","json"], default="str", help="output formats")
    PARSER.add_argument("sgrammar", metavar="sgrammar" , help="Grammar")
    ARGS = PARSER.parse_args()
    import sys
    DEBUGLEVEL = ARGS.debuglevel or logging.WARNING
    
    logging.basicConfig(level = DEBUGLEVEL)
    try:
        result = validate(**vars(ARGS))
    except EOFError:
        sys.exit(0)
    if not result:
        sys.exit(-1)
    sys.exit(0)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""Functions Library"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
try:
    # 3.x name
    import configparser
except ImportError:
    # 2.x name
    import ConfigParser as configparser
LOG = logging.getLogger("Storage.Directory.Function")

def _readBoardFileRightSideArgs(thestring, basegtname):
    """Reads input or output definition, and splits it accordingly"""
    from pydsl.Memory.File.BoardSection import BoardConnectionDefinition
    finallist = []
    tmplist = thestring.split(",")
    for gt in tmplist:
        if gt.count(".") == 2:
            tmpgt = gt.split(".")
            gtcondef = BoardConnectionDefinition(basegtname, tmpgt[0], tmpgt[1], tmpgt[2])
        else:
            raise SyntaxError
        finallist.append(gtcondef)
    return finallist

def sectionToBoardDefinition(configparser, sectionname):
    """Create a Transformer definition object from file"""
    items = configparser.items(sectionname)
    LOG.debug("sectionToBoardDefinition: Item List: " + str(items))
    myinput = _readBoardFileRightSideArgs(configparser.get(sectionname, "input"), sectionname)
    output = _readBoardFileRightSideArgs(configparser.get(sectionname, "output"), sectionname)
    mytype = configparser.get(sectionname, "type")
    from pydsl.Memory.File.BoardSection import BoardDefinitionSection
    return BoardDefinitionSection(sectionname, mytype, myinput, output)

def parseRegularSections(configparser):
    definitionlist = []
    for section in configparser.sections():
        definitionlist.append(sectionToBoardDefinition(configparser, section))
    return definitionlist

def load_board_file(filename):
    config = configparser.ConfigParser()
    config.read(filename)
    if len(config.sections()) == 0:
        from pydsl.Exceptions import BadFileFormat
        raise BadFileFormat(filename)
    GTDefinitionList = parseRegularSections(config)
    from pydsl.Function.Board import Board
    return Board(GTDefinitionList) 



def function(inputdic, inputgt, outputgt):
    if not inputdic["input"]:
        return {}
    numberstr = inputdic["input"]
    number = int(numberstr)
    hexstr = "%X" % number
    return {"output":hexstr}


iclass = "PythonTransformer"
inputdic = {"input":"integer"}
outputdic = {"output":"hex"}



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.


"""Storage class"""


__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"


import contextlib
import shelve
import logging
LOG = logging.getLogger(__name__)
from pydsl.Memory.Memory import Memory

class ShelveStorage(Memory):
    """Memory implementation using python shelve"""
    def __init__(self, dbname, allowedclass = None):
        Memory.__init__(self)
        self.identifier = dbname
        #store shelve path list
        #load/create each shelve path
        from pydsl.Config import GLOBALCONFIG
        if GLOBALCONFIG.persistent_dir:
            self.filename = GLOBALCONFIG.persistent_dir + "/" + dbname
        import threading
        self.lock = threading.Lock()
        self.allowedclass = allowedclass
    
    def load(self, name):
        return self.__getitem__(name)
    
    def __getitem__(self, name):
        with self.lock:
            a = shelve.open(self.filename)
            with contextlib.closing(a):
                result = a[name]
                return result
        
    def __iter__(self):
        self.index = 0
        with self.lock:
            a = shelve.open(self.filename)
            with contextlib.closing(a):
                self.cache = a.keys() #TODO: should return full dict, not only names
        return self

    def next(self):
        try:
            result = self.cache[self.index]
        except IndexError:
            raise StopIteration
        self.index += 1
        return result
    
    def save(self, instance, identifier):
        if self.allowedclass and not isinstance(instance, self.allowedclass):
            raise TypeError
        with self.lock:
            a = shelve.open(self.filename)
            with contextlib.closing(a):
                a[str(identifier)] = instance
        
    def __delitem__(self, element):
        with self.lock:
            a = shelve.open(self.filename)
            with contextlib.closing(a):
                del a[element]
        
    def __contains__(self, element):
        with self.lock:
            a = shelve.open(self.filename)
            with contextlib.closing(a):
                return element in a
        
    def provided_iclasses(self):
        if self.allowedclass:
            return [self.allowedclass.__class__.__name__]
        return [] #FIXME: Default implementation should include all classes





#!/usr/bin/python
# -*- coding: utf-8 -*-

"""Prints a parsetree"""

def symbollist_print(sl):
    resultlist = []
    for element in sl:
        resultlist.append(element.name)
    return ",".join(resultlist)
    

def tree_to_dot(stt):
    output = "digraph graphname \r{ "
    names, relations = aux_stt_dot(stt.first_item)
    for name in names:
        output += '"' + name + '"\r'
    for relation in relations: 
        output += '"' + relation[0] + '" -> "' + relation[1] + "\"\r"
    output += "}"
    return output

def treenode_to_name(treenode):
    result = treenode.word.string
    if len(treenode.rightside) == 1:
        result += " " + str(treenode.rightside[0]) 
    result += " (" + str(treenode.leftpos) + "," + str(treenode.rightpos) + ")"
    return result 

def aux_stt_dot(stt, names = [], relations = []):
    currentname = treenode_to_name(stt)
    if not str(currentname) in names:
        names.append(currentname)
    for child in stt.childlist:
        relations.append((currentname, treenode_to_name(child))) #+ symbollist_print(child.production.rightside)))
        aux_stt_dot(child, names, relations) 
    return names,relations
    

def function(inputdic, inputgt, outputgt):
    grammarname =  inputdic['grammar']
    from pydsl.Memory.Loader import load_parser
    grammar = load_parser(grammarname)
    result = grammar.get_trees(inputdic['string'])
    #print(result)
    #print(result[0])
    result = result[0]
    return {"output":tree_to_dot(result)}


inputdic = {"grammar":"cstring", "cstring":"cstring"}
outputdic = {"output":"cstring"}
iclass = "PythonTransformer"



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""Board Definition classes"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

class BoardConnectionDefinition(object):
    """Transformer connection definition"""
    def __init__(self, basename, source, gtname, destination):
        self.source = basename
        self.sourcechannel = source
        self.destination = gtname
        self.destinationchannel = destination

    def __str__(self):
        result = "<BoardConnectionDefinition: " + self.source + " " + self.sourcechannel
        result += " -> " + self.destination + ":" + self.destinationchannel
        return result

class BoardDefinitionSection(object):
    """Board and its connections definition"""
    def __init__(self, name, gttype, inputconnections, outputconnections):
        self.name = name
        self.type = gttype
        self.inputConnectionDefinitions = inputconnections #each element is a BoardConnectionDefinition
        self.outputConnectionDefinitions = outputconnections #each element is a BoardConnectionDefinition

    def __str__(self):
        result = "<BoardDefinitionSection "
        result += self.name + ", " + self.type 
        for con in self.inputConnectionDefinitions:
            result += str(con) 
        result += " - "
        for con in self.outputConnectionDefinitions:
            result += str(con) 
        result += ">"
        return result




def function(inputdic, inputgt, outputgt):
    palabras = str(inputdic["input"])
    return {"output":palabras.upper()}


iclass = "PythonTransformer"
inputdic = {"input":"cstring"}
outputdic = {"output":"cstring"}



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""Boards"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
LOG = logging.getLogger(__name__)

class Board(object):
    """A Transformer where you can call other Transformer. Doesn't perform any computation"""

    def __init__(self, gtenvdefinitionslist):
        self._hostT = {}
        self.__GTDefinitionlist = gtenvdefinitionslist #list to put every gt envdefinition
        self.__loadTfromDefinitionList()
        self.__inputGTDict = {} #{"channelname":GTinstance,} #Inner GT that receives input
        self.__outputGTDict = {} #{"channelname":GTinstance,}
        self.connectionsdict = {}
        self.__connectAllGTs()
        self.__extractExternalChannelGrammarsFromDefinitions()

    @property
    def inputchanneldic(self):
        #FIXME This should be the same that a HostChannel. It is here because it has the same length
        return self.__inputGTDict

    @property
    def summary(self):
        from pydsl.Abstract import InmutableDict
        #TODO: inputs and outputs
        result = {"iclass":"Board"}
        return InmutableDict(result)

    def __call__(self, inputdict):
        LOG.debug(" received dic:" + str(inputdict))
        resultstack = {}
        calldict = {}
        if not inputdict:
            LOG.error("No input")
            return None
        for channel, strcontent in inputdict.items():
            LOG.debug("Board Receiving: " + channel + " " + str(strcontent))
            #self.__sendToChannel(channel, rand, strcontent)
            gtname, keyname = self.__inputGTDict[channel]
            if not gtname in calldict:
                calldict[gtname] = {}
            calldict[gtname][keyname] = strcontent
        for gtname, functinput in calldict.items():
            resultstack[gtname] = self._hostT[gtname](functinput)
        #prepare extendedtuplelist

        extendedtuplelist = []
        for gtname, results in self.connectionsdict.items():
            for sourcechannel, duple in results.items():
                #sourcename, sourcechannel, destinationchannel, destination, destinationame
                extendedtuplelist.append((gtname, sourcechannel, duple[0], duple[1], duple[2]))
        change = True
        while change:
            change = False
            for _, _, _, curgt, curgtname in extendedtuplelist:
                if curgtname in resultstack:
                    continue
                curinputs = {}
                for sourcename, sourcechannel, destinationchannel, destination, destinationname in extendedtuplelist:
                    if not destination:
                        continue
                    if not destinationname in curinputs:
                        curinputs[destinationname] = {}
                    if sourcename in resultstack:
                        curinputs[destinationname][destinationchannel] = resultstack[sourcename][sourcechannel]
                if curgtname in curinputs and len(curinputs[curgtname]) == len(curgt.inputchanneldic):
                    change = True
                    #TODO:Delete used elements from resultstack
                    resultstack[curgtname] = self._hostT[curgtname](curinputs[curgtname])
            finaldict = {}
            for key, value in self.__outputGTDict.items():
                gtname, channelname = value
                finaldict[key] = resultstack[gtname][channelname]
            return finaldict
                

    def __extractExternalChannelGrammarsFromDefinitions(self):
        """Extracts grammars from definition.
        generated channel must be connected to outside elements"""
        #{"part1":["part2","part3"],
        # "part2":....
        # "partn":...}

        #Store initials and ends in lists
        #initials: ["part1"]
        #ends: ["partn"]

        for definition in self.__GTDefinitionlist:
            for gtcondef in definition.inputConnectionDefinitions:
                if gtcondef.destination == "Main":
                    self.__inputGTDict[gtcondef.destinationchannel] = (gtcondef.source, gtcondef.sourcechannel) #Prepares self.__inputGTDict
            for gtcondef in definition.outputConnectionDefinitions:
                if gtcondef.destination == "Main":
                    self.__outputGTDict[gtcondef.destinationchannel] = (gtcondef.source, gtcondef.sourcechannel) #Prepares self.__outputGTDict

    def __loadTfromDefinitionList(self):
        """GTDefinitions -> Instances"""
        from pydsl.Memory.File.BoardSection import BoardDefinitionSection
        auxnametype = {}
        for definition in self.__GTDefinitionlist:
            if not isinstance(definition, BoardDefinitionSection):
                raise TypeError
            gtname = definition.name
            gttype = definition.type
            auxnametype[gtname] = gttype
        self._initHostT(auxnametype)
        LOG.debug("__loadTfromDefinitionList: loaded __GTs: " + str(self._hostT.keys()))

    def __connectAllGTs(self):
        """Connect all instances"""
        for definition in self.__GTDefinitionlist:
            gtname = definition.name
            outputgrammarlist = definition.outputConnectionDefinitions
            self.connectionsdict[gtname] = {}
            for gtcondef in outputgrammarlist:
                hostt = None     
                if gtcondef.destination != "Main":
                    hostt = self._hostT[gtcondef.destination]
                self.connectionsdict[gtname][gtcondef.sourcechannel] = (gtcondef.destinationchannel, hostt, gtcondef.destination)

    def __str__(self):
        result = "<B "
        result += "input: " + str(self.inputchanneldic)
        result += "output: " + str(self.outputchanneldic)
        result += "inputGT: " + str(self.__inputGTDict)
        result += "outputGT: " + str(self.__outputGTDict)
        result += "connections: " + str(self._connections)
        result += ">"
        return result

    def _initHostT(self, namedic):
        """Inits aux GTs. if a requested aux GT isn't connected, This function will create them"""
        from pydsl.Memory.Loader import load
        for title, gttype in namedic.items():
            self._hostT[title] = load(gttype) 
            LOG.debug("loaded " + str(title) + "auxT")




#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.


"""
Calls a transformer
"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
from pydsl.Interaction.Shell import parse_shell_dict, open_files_dict, StreamFileToTransformerInteraction, save_result_to_output
from pydsl.Exceptions import BadFileFormat
from pydsl.Memory.Loader import load
LOG = logging.getLogger(__name__)

#FIXME Some parts of this class should call separated functions. This code is a mess
def translate(transformer = None, expression = None, inputfiledic = None, outputfiledic = None, inputstreamdic = None, pipemode = None, **kwargs):
    """Read input file contents, creates grammar and transform objects, create connections, 
    and afterwards reads required input/launch main loop"""
    #Generating and connecting output
    #listen to user, open read file, or other
    #configure output, write file, or other
    #print self._opt
    mainfunc = load(transformer) 
    if expression and  outputfiledic: #input type: expression #output: file
        inputdic = parse_shell_dict(expression)
        if not isinstance(inputdic, dict):
            raise TypeError
        outputdic = parse_shell_dict(outputfiledic)
        resultdic = mainfunc(inputdic)
        save_result_to_output(resultdic, outputdic)
        return resultdic
    elif expression and not outputfiledic:
        myexpression = parse_shell_dict(expression)
        result = mainfunc(myexpression)
        if result:
            for key in result.keys():
                result[key] = str(result[key])
        print(result)
        return result #FIXME: this is the only condition that returns a result. Because of tests
    elif inputstreamdic and outputfiledic:
        interactor = StreamFileToTransformerInteraction(mainfunc, parse_shell_dict(inputstreamdic), parse_shell_dict(outputfiledic))
        interactor.start()
    elif inputfiledic and outputfiledic:
        inputdic = parse_shell_dict(inputfiledic)
        outputdic = parse_shell_dict(outputfiledic)
        stringdic = open_files_dict(inputdic)
        resultdic = mainfunc(stringdic)
        save_result_to_output(resultdic, outputdic)
        return resultdic
    elif pipemode:
        assert(len(mainfunc.inputchanneldic) == 1)
        assert(len(mainfunc.outputchanneldic) == 1)
        inputname = list(mainfunc.inputchanneldic.keys())[0]
        outputname = list(mainfunc.outputchanneldic.keys())[0]
        interactor = StreamFileToTransformerInteraction(mainfunc, {inputname:"stdin"} , {outputname:"stdout"})
        interactor.start()
    elif not inputfiledic and not outputfiledic and not expression:
        from pydsl.Interaction.Shell import CommandLineToTransformerInteraction
        interactor = CommandLineToTransformerInteraction(mainfunc)
        interactor.start()
    else:
        raise Exception
    return True

if __name__ == "__main__":
    import argparse
    TUSAGE = "usage: %(prog)s [options] transformername"
    PARSER = argparse.ArgumentParser(usage = TUSAGE)
    PARSER.add_argument("-d", "--debuglevel", action="store", type=int, dest="debuglevel", help="Sets debug level")
    PARSER.add_argument("-is", "--inputstreams", action="store", dest="inputstreamdic", help="input streams filename dict")
    PARSER.add_argument("-i", "--inputfile", action="store", dest="inputfiledic", help="input filename dict")
    PARSER.add_argument("-o", "--outputfile", action="store", dest="outputfiledic", help="output filename dict")
    PARSER.add_argument("-e", "--expression", action="store", dest="expression", help="input expression")
    PARSER.add_argument("-p", "--pipe", action="store_true", dest="pipemode", help="Pipe interaction mode")
    PARSER.add_argument("transformer", help="Transformer name")
    ARGS = PARSER.parse_args()
    import sys
    if (ARGS.outputfiledic and not ARGS.expression) and (ARGS.outputfiledic and not ARGS.inputfiledic):
        PARSER.error("options -o and -e or -i must be together")
    DEBUGLEVEL = logging.WARNING
    if ARGS.debuglevel:
        DEBUGLEVEL = ARGS.debuglevel
    logging.basicConfig(level = DEBUGLEVEL)
    #except KeyError as le:
    #    print("Unable to load " + str(le))
    #    sys.exit(1)
    #else:
    #    print(TUSAGE)
    #    sys.exit(0)
    try:
        result = translate(**vars(ARGS))
    except EOFError:
        sys.exit(0)
    except BadFileFormat:
        print("Error reading input file")
        sys.exit(1)
    if not result:
        sys.exit(-1)
    sys.exit(0)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""Production rules"""

from pydsl.Grammar.Symbol import Symbol, TerminalSymbol
from pydsl.Grammar.Definition import GrammarDefinition

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"


class Production(object):
    def __init__(self, leftside, rightside):
        #Left side must have at least one non terminal symbol
        for element in rightside:
            if not isinstance(element, Symbol):
                raise TypeError
        self.leftside = leftside
        self.rightside = rightside

    def __str__(self):
        """Pretty print"""
        leftstr = " ".join([ x.name for x in self.leftside])
        rightstr = " ".join([ str(x) for x in self.rightside])
        return leftstr + "::=" + rightstr

    def __eq__(self, other):
        try:
            if len(self.leftside) != len(other.leftside):
                return False
            if len(self.rightside) != len(other.rightside):
                return False
            for index in range(len(self.leftside)):
                if self.leftside[index] != other.leftside[index]:
                    return False
            for index in range(len(self.rightside)):
                if self.rightside[index] != other.rightside[index]:
                    return False
        except AttributeError:
            return False
        return True

    @property
    def name(self):
        if len(self.leftside) != 1:
            raise Exception
        return self.leftside[0].name


class BNFGrammar(GrammarDefinition): #Only stores a ruleset, and methods to ask properties or validity check 
    def __init__(self, initialsymbol, fulllist, options=None):
        self._initialsymbol = initialsymbol
        for rule in fulllist:
            if fulllist.count(rule) >1:
                raise ValueError
        self.fulllist = fulllist
        if not options: options = {}
        self.options = options

    @property
    def productionlist(self):
        return [x for x in self.fulllist if isinstance(x, Production)]

    @property
    def terminalsymbollist(self):
        return [x for x in self.fulllist if isinstance(x,
            TerminalSymbol)]

    @property
    def first(self):
        return [x.first for x in self.terminalsymbollist]

    @property
    def left_recursive(self):# -> bool:
        """Tests if exists left recursion"""
        #TODO
        raise NotImplementedError

    @property
    def right_recursive(self):# -> bool:
        """Tests if exists right recursion"""
        #TODO
        raise NotImplementedError

    @property
    def is_abstract(self):
        """Returns true if the grammar contains an unknown symbol"""
        from pydsl.Grammar.Symbol import UnknownSymbol
        return UnknownSymbol in self.fulllist

    def __eq__(self, other):
        if not isinstance(other,BNFGrammar):
            return False
        if self._initialsymbol != other.initialsymbol:
            return False
        for index in range(len(self.productionlist)):
            if self.productionlist[index] != other.productionlist[index]:
                return False
        return True

    #def __getitem__(self, index):
    #    for rule in self.productionlist:
    #        if rule.name == index:
    #            return rule
    #    raise IndexError

    @property
    def initialsymbol(self):
        return self._initialsymbol

    @property
    def main_production(self):
        """Returns main rule"""
        for rule in self.productionlist:
            if rule.leftside[0] == self._initialsymbol:
                return rule
        raise IndexError

    def getProductionsBySide(self, symbollist, side = "left"):
        result = []
        for rule in self.productionlist: #FIXME Is iterating over production only
            part = None
            if side == "left":
                part = rule.leftside
            elif side == "right":
                part = rule.rightside
            else:
                raise KeyError
            valid = True
            if len(part) != len(symbollist):
                valid = False
            for ruleindex in range(len(part)):
                if part[ruleindex] != symbollist[ruleindex]:
                    valid = False
                    break
            if valid:
                result.append(rule)
        if not result:
            raise IndexError("Symbol: %s" % str([str(x) for x in symbollist]))

        return result

    def getProductionByBothSides(self, leftsymbollist, rightsymbollist):
        for rule in self.productionlist:
            valid = True
            if len(rule.leftside) != len(leftsymbollist):
                continue
            if len(rule.rightside) != len(rightsymbollist):
                continue
            for ruleindex in range(len(rule.leftside)):
                if leftsymbollist[ruleindex] != rule.leftside[ruleindex]:
                    valid = False
                    break
            if not valid:
                continue
            for ruleindex in range(len(rule.rightside)):
                if rightsymbollist[ruleindex] != rule.rightside[ruleindex]:
                    valid = False
                    break
            if not valid:
                continue
            if valid:
                return rule
        raise IndexError

    def getSymbols(self):
        """Returns every symbol"""
        symbollist = []
        for rule in self.productionlist:
            for symbol in rule.leftside + rule.rightside:
                if symbol not in symbollist:
                    symbollist.append(symbol)
        symbollist += self.terminalsymbollist
        return symbollist

    def getProductions(self):
        return self.productionlist

    def getTerminalSymbols(self):
        """Returns a list with every terminal symbol """
        return self.terminalsymbollist

    def getProductionIndex(self, rule):
        return self.productionlist.index(rule)

    def __str__(self):
        return str(list(map(str,self.productionlist)))

def create_non_terminal_production(productionset, terminalsymbol):
    """Creates a new nonterminalrule for terminalsymbol
    A -> B,nt 
    is changed to: 
    A -> B,C  
    C -> nt
    """
    #walks through all rules looking for our terminalsymbol in rightside
    from .Symbol import NonTerminalSymbol
    productionstochange = []
    for index, production in enumerate(productionset[:]): #copy
        if terminalsymbol in production.rightside:
            for symbol in production.rightside:
                if isinstance(symbol, NonTerminalSymbol):
                    productionstochange.append(production)
                    del productionset[index]
                    break
    #Inserts new rule
    newsymbol = NonTerminalSymbol(str(terminalsymbol))
    newproduction = Production(newsymbol, terminalsymbol)
    productionset.append(newproduction)
    #modifies required rules to link with new production
    for production in productionstochange:
        newproduction = Production(production.leftside[:], production.rightside[:])
        for index, element in enumerate(production.rightside):
            if element == terminalsymbol:
                del newproduction.rightside[index]
                newproduction.rightside.insert(index, newsymbol)
        productionset.append(newproduction)
    return productionset

def reduce_non_terminal_production(productionset, uselesssymbol):
    """search for production that only have nonterminalsymbol at right, subst nonterminal from rules at right and  remove the rule that have it at left"""
    production = productionset.getProductionsBySide(uselesssymbol, "right")
    productionset.remove(production)
    validterminalsymbol = production.leftside[0] #usefulterminalymbol
    productionlist = productionset.getProductionsBySide(uselesssymbol)
    saverule = False

    if len(productionlist) > 1:
        saverule = True
        #several rules with uselesssymbol at left

    for production in productionset:    
        if uselessnonterminal in production.rightside:
            #adds those who have nonterminal at right 
            newproduction = production.copy()
            index = newproduction.index(uselessnonterminal)
            del newproduction[index]
            newproduction.insert(index, validterminalsymbol)
            if not saverule:#Si no habia otra produccion
                productionset.remove(production)


def split_production(productionset, production):
    from .Symbol import NonTerminalSymbol
    if len(production.rightside) < 2:
        return productionset
    index = 0
    productionset.remove(production)
    newrightside = []
    while index < len(production) - 2:
        firsttwo = production.rightside[index:index+2]
        newsymbol = NonTerminalSymbol(production.leftside[0].name + "E")
        newrightside.append(newsymbol)
        newproduction = Production(newsymbol, firsttwo)
        productionset.append(newproduction)
        index += 2
    newrightside += production.rightside[index:]
    newproduction = Production(production.leftside, newrightside)
    productionset.append(newproduction)
    return productionset


def chomsky_normal_form(productionset):# -> BNFGrammar:
    """TODO Check is Epsilon free"""
    ntermslist = []
    from .Symbol import NonTerminalSymbol
    #STEP 1: A -> B,c 
    #Terminal and nonterminal at right side 
    for production in productionset:
        termlist = 0
        ntermlist = 0
        for element in production.rightside:
            if isinstance(element, TerminalSymbol):
                termlist.append(element)
            elif isinstance(element, NonTerminalSymbol):
                ntermlist.append(element)
        if termlist and ntermlist:
            ntermslist += termlist
    for terminal in ntermlist:
        productionset = create_non_terminal_production(productionset, terminal)
    #STEP 2: useless nonterminal
    uselessnonterms = []
    for production in productionset:
        if len(production.rightside) == 1 and isinstance(production.rightside[0], NonTerminalSymbol):
            uselessnonterms.append(production.rightside[0])
    for nonterm in uselessnonterms:
        productionset = reduce_non_terminal_production(productionset, nonterm)

    #STEP 3:rightside is too long 
    longproductions = []
    for production in productionset:
        if len(production.rightside) > 2:
            longproductions.append(production)
    for production in longproductions:
        productionset = split_production(productionset, nonterm)

    return productionset



#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (C) 2008-2013 Nestor Arocha

"""
Token example grammar.   
accepts only "1+1"
"""

from pydsl.Grammar.Symbol import StringTerminalSymbol as _CTS, WordTerminalSymbol as _WTS, NonTerminalSymbol as _NTS
from pydsl.Grammar.BNF import Production as _NTP

_br = "max"


_symbol1 = _WTS("one", {"grammarname":"integer"}, _br)
_symbol2 = _CTS("+")
initialsymbol = _NTS("exp") 

_rule1 = _NTP([initialsymbol], [_symbol1, _symbol2, _symbol1])
fulllist = [_rule1, _symbol1, _symbol2]
iclass = "SymbolGrammar"



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

""" Channels: Where words are sent and received """

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"


import logging
LOG = logging.getLogger(__name__)

def _loadGrammarTools(originaldic):
    """Converts {"channelname","type"} into {"channelname",instance}"""
    from pydsl.Memory.Loader import load_grammar_tool
    result = {}
    for key in originaldic:
        result[key] = load_grammar_tool(str(originaldic[key]))
    return result

class HostChannel(object):
    """A class that contains input and output string-named channels. Each channel must contain a Type object
    Any class which inherites from this must also inherit from HostChannel
    """
    def __init__(self, inputtypedict, outputtypedict):
        for key in inputtypedict:
            if not isinstance(key, str):
                raise TypeError
        for key in outputtypedict:
            if not isinstance(key, str):
                raise TypeError
        self.inputdefinition = inputtypedict
        self.outputdefinition = outputtypedict
        self.__inputchanneldic = _loadGrammarTools(inputtypedict)
        self.__outputchanneldic = _loadGrammarTools(outputtypedict)

    @property
    def inputchanneldic(self):
        return self.__inputchanneldic

    @property
    def outputchanneldic(self):
        return self.__outputchanneldic



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""pydsl element Interaction"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import sys
import logging
LOG = logging.getLogger(__name__)

promptstr = "Insert data (q to exit)"


def escapedsplitby(inputstring, separator):
    """Splits inputstring with separator and using "\" as a escape character"""
    #TODO check alternatives (shlex and csv)
    result = []
    while inputstring[0] == separator:
        inputstring = inputstring[1:]
    lastindex = 0
    index = 0
    while index < len(inputstring):
        if inputstring[index] == "\\" and inputstring[index + 1] == separator:
            inputstring = inputstring[:index] + inputstring[index + 1:]
            index += 1
            continue
        elif inputstring[index] == "\\" and inputstring[index + 1] == "\\":
            inputstring = inputstring[:index] + inputstring[index + 1:]
            index += 1
            continue
        elif inputstring[index] == separator:
            result.append(inputstring[lastindex:index])
            while inputstring[index] == separator:
                index += 1
            lastindex = index
            continue
        else:
            index += 1
            continue
    result.append(inputstring[lastindex:])
    return result


def parse_shell_dict(inputstring): # -> dict:
    """Parses commandline input dicts. Example: a:b,c:d,e:f."""
    result = {}
    arglist = escapedsplitby(inputstring, ",")
    for arg in arglist:
        pair = escapedsplitby(arg, ":")
        result[pair[0]] = pair[1]
    return result


def open_files_dict(inputdic): # -> dict:
    """Converts a all str filename values into file objects"""
    result = {}
    for channel, filename in inputdic.items():
        with open(filename, 'rb') as f:
            result[channel] = f.read()
    return result


def save_result_to_output(resultdic, outputdic):
    """Saves results dict to files in output values.
    Both dicts must have the same keys"""
    if not resultdic:
        raise Exception("Error: " + str(resultdic))
    for key in outputdic:
        if not key in resultdic:
            raise KeyError("No output channel detected")
    for key in outputdic:
        if outputdic[key] == "stdout":  # print to screen
            print(resultdic[key])
        else:
            with open(outputdic[key], 'w') as currentfile:  # print to file
                currentfile.write(str(resultdic[key]))


class CommandLineToTransformerInteraction(object):
    """Shell interaction for functions"""
    def __init__(self, gt):
        self._tinstance = gt

    def start(self):
        print("Input: " + ",".join(self._tinstance.inputchanneldic.keys()))
        value = self._getInput()
        while value is not None:
            resultdic = self._tinstance(value)
            if not resultdic:
                print(str(resultdic) + "\n")
            else:
                for key in resultdic.keys():
                    try:
                        resultdic[key] = str(resultdic[key])
                    except UnicodeDecodeError:
                        resultdic[key] = "Unprintable"
                if len(resultdic) == 1:
                    print(str(list(resultdic.values())[0]) + "\n")
                else:
                    print(str(resultdic) + "\n")
            value = self._getInput()
        print("Bye Bye")

    def _getInput(self):
        print(promptstr)
        var = "Anything"
        while True:
            inputdic = {}
            try:
                for channel in self._tinstance.inputchanneldic.keys():
                    var = input(channel + ":\n")
                    if var == "q":
                        return None
                    inputdic[channel] = var
            except (SyntaxError, NameError, TypeError):
                pass
            for channel in self._tinstance.inputchanneldic.keys():
                if channel not in inputdic:
                    print('Invalid python dict. Example: {"key":"value1",...}')
                    var = input(promptstr)  # TODO use AST
                continue
            return inputdic


class StreamFileToTransformerInteraction(object):
    """Write to file n times"""
    def __init__(self, gt, inputfiledic, outputfiledic=None):
        if not outputfiledic: outputfiledic = {}
        self._tinstance = gt
        self._inputfiledic = inputfiledic
        self._outputfiledic = outputfiledic

    def start(self):
        inputdic = {}
        stringdic = {}
        for channel, filename in self._inputfiledic.items():
            if filename == "stdin":
                inputdic[channel] = "stdin"
                stringdic[channel] = "stdin"
            else:
                inputdic[channel] = open(filename, 'r')
                stringdic[channel] = inputdic[channel].readline()
        myand = lambda a, b: a and b
        from functools import reduce
        while reduce(myand, stringdic.values()):
            stringdiccopy = dict(stringdic)
            endofstdin = False
            for channel, content in stringdiccopy.items():
                if content == "stdin":
                    line = sys.stdin.readline()
                    if not line:
                        endofstdin = True
                    else:
                        stringdic[channel] = line.rstrip('\n')
                    break
            if endofstdin:
                break  # No new line received
            resultdic = self._tinstance(stringdic)
            self.__showOutput(resultdic)
            for channel, filename in self._inputfiledic.items():
                if filename == "stdin":
                    stringdic[channel] = "stdin"
                else:
                    stringdic[channel] = inputdic[channel].readline()
                    if not stringdic[channel]:
                        break
        for key, filehandler in inputdic.items():
            if key != "input" and filehandler != "stdin":
                filehandler.close()

    def __showOutput(self, resultdic):
        #TODO: check if is an error. if stdout is used, use stderr
        if not resultdic:
            sys.stderr.write(str(resultdic) + "\n")
        elif not self._outputfiledic:
            #No output information, stdout assumed
            for key in resultdic:
                print(resultdic[key].string.strip())
        else:
            for key in self._outputfiledic:
                if not key in resultdic:
                    raise Exception("No output channel detected")
            for key in self._outputfiledic:
                if self._outputfiledic[key] == "stdout":  # print to screen
                    print(resultdic[key].string.strip())
                else:  # print to file
                    with open(self._outputfiledic[key], 'a') as currentfile:
                        currentfile.write(resultdic[key].string + "\n")



#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (C) 2008-2013 Nestor Arocha

def function(inputdic, inputgt, outputgt):
    filename = inputdic["input"].string
    with open(filename, 'r') as f:
        result = f.read()
        result = str(result)
        return {"output":result}


iclass = "PythonTransformer"
inputdic = {"input":"unixPathGrammar"}
outputdic = {"output":"Hex"}



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.



"""Syntax Directed Transformers"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
LOG = logging.getLogger(__name__)

class SyntaxDirectedTransformer(object):
    def __init__(self, inputgrammar, outputgrammar, blockdic):
        self.blockdic = blockdic
        if isinstance(inputgrammar, str):
            from pydsl.Memory.Loader import load
            inputgrammar = load(inputgrammar)
        from pydsl.Grammar.Parser.RecursiveDescent import RecursiveDescentParser
        self.inputgrammar = inputgrammar
        self.outputgrammar = outputgrammar
        self.parser = RecursiveDescentParser(inputgrammar)
        self.blockdic = blockdic

    def __parseSymbolTokenTree(self, stt):
        """Returns a tokenlist"""
        #productionruleset = list(self.inputchanneldic.values())[0].productionset
        from pydsl.Grammar.Symbol import TerminalSymbol
        if isinstance(stt.production, TerminalSymbol):
            return stt.content
        childlist = []
        for child in stt.childlist:
            childlist.append(self.__parseSymbolTokenTree(child))
        if not str(stt.production) in self.blockdic and len(childlist) == 1:
            return childlist[0]
        return self.blockdic[str(stt.production)](childlist)

    def __call__(self, word):
        stt = self.parser.get_trees(word)[0]
        return self.__parseSymbolTokenTree(stt)

    @property
    def summary(self):
        return {"iclass":"SyntaxDirectedTransformer", "input":self.inputgrammar, "output":self.outputgrammar }



#!/usr/bin/python
# -*- coding: utf-8 -*-

def function(inputdic, inputgt, outputgt):
    mainstr =  inputdic['input']
    pointer = 0
    memory = {}
    index = 0
    result = ""
    while index in range(len(mainstr)):
        x = mainstr[index]
        if x == '>':
            pointer += 1
            index += 1
        elif x == '<':
            pointer -= 1
            index += 1
        elif x == '+':
            if not pointer in memory:
                memory[pointer] = 0
            memory[pointer] += 1
            index += 1
        elif x == '-':
            if not pointer in memory:
                memory[pointer] = 0
            memory[pointer] -= 1
            index += 1
        elif x == '.':
            result += chr(memory[pointer])
            index += 1
        elif x == ',':
            memory[pointer] = ord(result[-1])
            index += 1
        elif x == '[':
            if not memory.get(pointer, 0):
                index += 1
                while mainstr[index] != ']':
                    index+=1
            index += 1
        elif x == ']':
            if memory.get(pointer, 0):
                index -= 1
                while mainstr[index] != '[':
                    index-=1
            index += 1
    return {"output":result}


inputdic = { "input":"brainfuck"}
outputdic = {"output":"cstring"}
iclass = "PythonTransformer"



#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (C) 2008-2013 Nestor Arocha

def function(inputdic, inputgt, outputgt):
    if not inputdic["input"]:
        return {}
    numberstr = inputdic["input"]
    decimalnumber = int(numberstr, 16)
    return {"output":str(decimalnumber)}


iclass = "PythonTransformer"
inputdic = {"input":"hex"}
outputdic = {"output":"integer"}



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""
Search Queries
"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"


from pydsl.Query import Query, QueryEquality, QueryInclusion, QueryGreaterThan

def str_to_memoryquery(string, escape = '\\'):# -> Query:
    """String a query"""
    #We parse de string and generate an equivalent list
    elementlist = [] 
    pos = 0
    currentword = ""
    while pos < len(string):
        if string[pos] == "&" and string[pos+1] == "&":
            elementlist.append(currentword)
            currentword = ""
            elementlist.append("&&")
            pos += 2
            continue
        elif string[pos] == "|" and string[pos+1] == "|":
            elementlist.append(currentword)
            currentword = ""
            elementlist.append("||")
            pos += 2
            continue
        elif string[pos] == "!":
            elementlist.append(currentword)
            currentword = ""
            elementlist.append("!")
            pos += 1
            continue
        elif string[pos] == "=":
            elementlist.append(currentword)
            currentword = ""
            elementlist.append("=")
            pos += 1
            continue
        elif string[pos] == "@":
            elementlist.append(currentword)
            currentword = ""
            elementlist.append("@")
            pos += 1
            continue
        elif string[pos] == ">":
            elementlist.append(currentword)
            currentword = ""
            elementlist.append(">")
            pos += 1
            continue
        elif string[pos] == escape: #insert next symbol into currentword
            currentword += string[pos] + string[pos+1]
            pos += 2
            continue
        else:
            currentword += string[pos]
            pos += 1
    elementlist.append(currentword)
    if len(elementlist) == 1: #Only one str received
        likestr = "/.*" + string + ".*/"
        return Query(QueryEquality("identifier",string))
    else:
        return Query(recursive_str_to_query(string))

def recursive_str_to_query(query_list): # -> QueryElement:
    from pydsl.Query import AndQueryOperator, OrQueryOperator, NotQueryOperator
    if "||" in query_list:
        index = query_list.index("||")
        term1 = recursive_str_to_query(query_list[:index])
        term2 = recursive_str_to_query(query_list[index+1:])
        return OrQueryOperator(term1, term2)
    if "&&" in query_list:
        index = query_list.index("&&")
        term1 = recursive_str_to_query(query_list[:index])
        term2 = recursive_str_to_query(query_list[index+2:])
        return AndQueryOperator(term1, term2)
    if "!" in query_list:
        index = query_list.index("!")
        term = recursive_str_to_query(query_list[index+2:])
        return NotQueryOperator(term)
    if query_list.count("=") == 1:
        pair = query_list.split("=")
        return QueryEquality(pair[0],pair[1])
    elif query_list.count("@") == 1:
        pair = query_list.split("@")
        return QueryInclusion(pair[0],pair[1])
    elif query_list.count(">") == 1:
        pair = query_list.split(">")
        return QueryGreaterThan(pair[0],pair[1])
    else:
        raise Exception

def dict_to_query(querydict): # -> Query:
    return Query(recursive_dict_to_query(querydict))

def recursive_dict_to_query(querydict, parentkey = None): # -> QueryElement:
    if isinstance(querydict, str):
        return querydict
    from pydsl.Query import QueryEquality, QueryInclusion, QueryGreaterThan, AndQueryOperator, NotQueryOperator, QueryPartial
    resultlist = []
    for key, value in querydict.items():
        if isinstance(value, dict):
            if len(value) == 1 and "$in" in value:
                resultlist.append(QueryInclusion(key, recursive_dict_to_query(value["$in"])))
            elif len(value) == 1 and "$gt" in value:
                resultlist.append(QueryGreaterThan(parentkey,recursive_dict_to_query(value["$gt"])))
            elif len(value) == 1 and "$not" in value:
                resultlist.append(QueryEquality(key,NotQueryOperator(recursive_dict_to_query(value["$not"]))))
            elif len(value) == 1 and "$part" in value:
                resultlist.append(QueryPartial(key, value["$part"]))
            else:
                resultlist.append(recursive_dict_to_query(value, key))
        elif isinstance(value, str):
            resultlist.append(QueryEquality(key,value))
    myfun = lambda x,y: AndQueryOperator(x,y)
    from functools import reduce
    return reduce(myfun, resultlist)
    




#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
LOG = logging.getLogger(__name__)

class GrammarTools(object):
    """Easy to access class that has members for checking, groups, tokenizing"""
    def __init__(self, gd):
        self.gd = gd
        self._checker = None

    def get_groups(self, information, propertyname = None):# -> list:
        """  
        returns [wordgroup1, wordgroup2] (list). A list of tokens.  provides dict like access.
        if propertyname == none: returns all input groups 
        """
        return []

    def groups(self): # -> set:
        """Returns a set with all groups names"""
        return set() 

    def enumerate(self): # -> set:
        """Yields a list of accepted values (if possible) else return {}"""
        return set() 

    def alphabet(self): # -> set:
        """Returns the alphabet of this grammar, a list of token (if possible) else return {}"""
        return set()

    def tokenize(self, information):
        """tokenize and iterates through all tokens of the input (from left to right)"""
        raise NotImplementedError

    def distance(self, input1, input2 = None):
        """Gives a distance score between two words according to this grammar. If input2 == None, means distance from input1 to valid"""
        raise NotImplementedError

    @property
    def minsize(self):
        raise NotImplementedError

    @property
    def maxsize(self):
        raise NotImplementedError

    def check(self, word):
        if not self._checker:
            from pydsl.Memory.Loader import load_checker
            self._checker = load_checker(self.gd)
        return self._checker.check(word)




#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""Symbols"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
LOG = logging.getLogger(__name__)

class Symbol(object):
    def __init__(self, name, weight): 
        self.name = name
        self._weight = weight

    def __eq__(self, other):
        if not isinstance(other, Symbol):
            return False
        return self.name == other.name

    def __ne__(self, other):
        """Operator != """
        return not self.__eq__(other)

    @property
    def weight(self):
        return self._weight

    def first(self, length = 1):
        """Returns the list of possible first elements"""
        raise NotImplementedError

class NonTerminalSymbol(Symbol):
    def __init__(self, name,  weight = 50):
        Symbol.__init__(self, name, weight)

    def __str__(self):
        return "<NonTS: " + self.name + ">"

    def __eq__(self, other):
        if not isinstance(other, NonTerminalSymbol):
            return False
        return self.name == other.name

class TerminalSymbol(Symbol): 
    def __init__(self, name, weight, boundariesrules): 
        Symbol.__init__(self, name, weight)
        if boundariesrules not in ("min","max","any","fixed"):
            raise TypeError
        self.boundariesrules = boundariesrules

    def check(self, data):# ->bool:
        """Checks if input is recognized as this symbol"""
        raise NotImplementedError


class StringTerminalSymbol(TerminalSymbol): #FIXME This class is equivalent to a StaticGrammarDefinition
    def __init__(self, string):
        if len(string) < 1:
            raise TypeError
        TerminalSymbol.__init__(self, "StrSymbol " + string, 99, "fixed")
        self.definition = string

    def check(self, tokenlist):# -> bool:
        return tokenlist == self.definition

    def __eq__(self, other):
        """StringTerminalSymbol are equals if definition and names are equal"""
        if isinstance(other, str):
            return self.definition == other
        if not isinstance(other, StringTerminalSymbol):
            return False
        if self.definition != other.definition:
            return False
        if self.name != other.name:
            return False
        return True

    def __len__(self):
        return len(self.definition)

    def first(self):
        return self.definition[0]

    def __str__(self):
        return "<StringTS: " + self.definition + ">"


class WordTerminalSymbol(TerminalSymbol):#boundariesrules: [max,min,fixedsize]
    def __init__(self, name, definitionrequirementsdic, boundariesrules):
        TerminalSymbol.__init__(self, name, 49, boundariesrules)
        self.grammarname = definitionrequirementsdic["grammarname"]
        self.__checker =  None 

    @property
    def checker(self):
        if self.__checker is None:
            from pydsl.Memory.Loader import load_checker
            self.__checker = load_checker(self.grammarname)
        return self.__checker

    def __eq__(self, other):
        if not isinstance(other, WordTerminalSymbol):
            return False
        if self.grammarname != other.grammarname:
            return False
        if self.name != other.name:
            return False
        return True

    def check(self, string):
        result =  self.checker.check(string)
        return result

    def __str__(self):
        return "<WordTS: " + self.grammarname + ">"


class UnknownSymbol(TerminalSymbol):
    def __init__(self):
        TerminalSymbol.__init__(self, "Unknown", 1, "any")

    def __eq__(self, other):
        return isinstance(other, UnknownSymbol)

    def check(self, data):
        return bool(data)

class NullSymbol(Symbol):
    def __init__(self):
        Symbol.__init__(self, "Null", 100)

    def __eq__(self, other):
        return isinstance(other, NullSymbol)

    def bool(self):
        return False

class EndSymbol(TerminalSymbol):
    pass



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""
Query and related classes
"""

#Current query class only works for indexable elements
#for concepts ands relations, other operators are required: ELEMENT HASA CONCEPT1 OR ELEMENT ISA CONCEPT4 OR ELEMENT PROPERTY CONCEPT5

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

class QueryElement(object):
    pass

class QueryTerm(QueryElement):
    """Any query Term"""
    def __init__(self, left, right):
        self.left = left
        self.right = right

    def __hash__(self):
        return hash(self.left) ^ hash(self.right)


class QueryEquality(QueryTerm):
    """ a = b. It can use a string or a regexp"""
    def __str__(self):
        return "<" + str(self.left) + "=" + str(self.right) + ">"

class QueryPartial(QueryTerm):
    """ a = b. It can use a string or a regexp"""
    def __str__(self):
        return "<" + str(self.left) + "=~" + str(self.right) + ">"


class QueryInclusion(QueryTerm):
    """ looks for an element within a list """
    pass

class QueryGreaterThan(QueryTerm):
    """associated with > symbol """
    pass

class BinaryOperator(QueryElement):
    def __init__(self, element1, element2):
        self.element1 = element1
        self.element2 = element2

class AndQueryOperator(BinaryOperator):
    def __str__(self):
        return "<" + str(self.element1) + "&&" + str(self.element2) + ">"

class OrQueryOperator(BinaryOperator):
    pass

class NotQueryOperator(QueryElement):
    def __init__(self, element):
        self.element = element

    def __str__(self):
        return "<!" + str(self.element) + ">"

class Query(object):
    """A generic query"""
    def __init__(self, content):
        self.content = content 

    def qand(self, element):
        self.content = AndQueryOperator(self.content, element)

    def qor(self, element):
        self.content = OrQueryOperator(self.content, element)

    def __str__(self):
        return"<Query:"+str(self.content)+">"




#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""
Show input's components according to a language
"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging


def print_errors(postnode):
    result = ""
    if not postnode.valid:
        result += "Error at production: " + str(postnode.production) + "(" + str(postnode.leftpos) + "," + str(postnode.rightpos) + ")"
        for child in postnode.childlist:
            result += print_errors(child) + "\n"
    return result

def parts(grammar, part, outputformat = None, expression = None, inputfile = None):
    from pydsl.Memory.Loader import load
    sgrammar = load(grammar) 
    if not part:
        print("Grouplist: " + "\n".join(sgrammar.groups()))
        return True
    if expression:
        myinput = expression
    elif inputfile:
        with open(inputfile, "rb") as f:
            myinput = f.read()
    else:
        raise Exception("No input method")
    result = sgrammar.get_groups(myinput, part)
    dataresults = []
    for left,right in result:
        dataresults.append(myinput[left:right])

    if outputformat == "str":
        for x in range(len(result)):
            print("(" + str(result[x][0]) + "," + str(result[x][1]) + ") " +  str(dataresults[x]))
        return True

    elif outputformat == "json":
        mylist = []
        for x in range(len(result)):
            mylist.append((result[x][0],result[x][1],dataresults[x]))
        import json
        print(json.dumps(mylist))
        return True


if __name__ == "__main__":
    import argparse
    TUSAGE = "usage: %(prog)s [options] [filename]"
    PARSER = argparse.ArgumentParser(usage = TUSAGE)
    PARSER.add_argument("-d", "--debuglevel", action="store", type=int, dest="debuglevel", help="Sets debug level")
    PARSER.add_argument("-i", "--inputfile", action="store", dest="inputfile", help="input filename ")
    PARSER.add_argument("-e", "--expression", action="store", dest="expression", help="input expression")
    PARSER.add_argument('-o', dest='outputformat',nargs='?', choices=["str","json"], default="str", help="output formats")
    PARSER.add_argument("sgrammar", metavar="sgrammar" , help="Grammar")
    PARSER.add_argument("part", nargs='?', metavar="part" , help="Part name")
    ARGS = PARSER.parse_args()
    DEBUGLEVEL = ARGS.debuglevel or logging.WARNING
    
    logging.basicConfig(level = DEBUGLEVEL)
    import sys
    try:
        result = parts(vars(**ARGS))
    except EOFError:
        sys.exit(0)
    if not result:
        sys.exit(-1)
    sys.exit(0)



#!/usr/bin/python
# -*- coding: utf-8 -*-

def function(inputdic, inputgt, outputgt):
    output = ""
    value1 = str(inputdic["int1"])
    value2 = str(inputdic["int2"])
    if value1 != "" and value2 != "": 
        output = str(int(value1) + int(value2))
    return {"int":output}

inputdic = {"int1":"integer", "int2":"integer"}
outputdic = {"int":"integer"}
iclass = "PythonTransformer"



#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (C) Mark Pilgrim

#Example from dive into python book
#Under GNU Free Documentation License

_romanNumeralMap = (('M',  1000),
                   ('CM', 900),
                   ('D',  500),
                   ('CD', 400),
                   ('C',  100),
                   ('XC', 90),
                   ('L',  50),
                   ('XL', 40),
                   ('X',  10),
                   ('IX', 9),
                   ('V',  5),
                   ('IV', 4),
                   ('I',  1))


def function(inputdic, inputgt, outputgt):
    output = ""
    result = ""
    if str(inputdic["input"]) != "": 
        value = int(str(inputdic["input"]))
        if value > 4000 or value < 1:
            output = "MMMM"
        else:
            for numeral, integer in _romanNumeralMap:
                while value >= integer:
                    result += numeral
                    value -= integer
            output = result
    return {"output":output}

iclass = "PythonTransformer"
inputdic = {"input":"integer"}
outputdic = {"output":"romanNumerals"}




def function(inputdic, inputgt, outputgt):
    liststr = inputdic["input"]
    return {"output":liststr.count(",") + 1}


iclass = "PythonTransformer"
inputdic = {"input":"list"}
outputdic = {"output":"integer"}



#!/usr/bin/python3

""" Memory management
   verbs:    i info
             s search
             l list elements
"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
from pydsl import VERSION
from pydsl.Config import GLOBALCONFIG

def search_pp(inputset: set, filterlist = None) -> str:
    """Search pretty print"""
    result = ""
    for element in inputset:
        for key in element:
            if filterlist is None or key in filterlist:
                if isinstance(element[key], tuple):
                    result += key + "-> " + str(element[key]) + " "
                else:
                    result += key + ": " + str(element[key]) + " "
        result += '\n'
    return result

def filterset(inputset: set, filterlist = None) -> set:
    if filterlist is None:
        return inputset #Don't filter at all
    from pydsl.Abstract import InmutableDict
    result = set()
    for element in inputset:
        telement = {}
        for key in element:
            if key in filterlist:
                telement[key] = element[key]
        result.add(InmutableDict(telement))
    return result


def info(identifier, outputformat):
    memorylist = GLOBALCONFIG.memorylist
    for memory in memorylist:
        if identifier in memory:
            instance = memory.load(identifier)
            break
    else:
        print("Not found")
        return False
    if instance:
        if hasattr(instance, "summary"):
            resultdic = instance.summary
        else:
            resultdic = {"instance":instance}
        if outputformat == "str":
            for key in resultdic:
                print(key + ": " + str(resultdic[key]))
        elif outputformat == "json":
            import json
            print(json.dumps(resultdic))
        else:
            print("Raw format not supported")
            return False
    else:
        if outputformat == "str":
            print(str(instance))
        elif outputformat == "raw":
            import sys
            sys.stdout.write(str(instance))
        else:
            print("JSON format not supported")
            return False
    return True

if __name__ == "__main__":
    import argparse
    TUSAGE = "usage: %(prog)s [options] verb [identifier]"
    PARSER = argparse.ArgumentParser(usage = TUSAGE)
    PARSER.add_argument("-d", "--debuglevel", action="store", type=int, dest="debuglevel", help="Sets debug level")
    PARSER.add_argument('-o', dest='outputformat',nargs='?', choices=["str","json","raw"], default="str", help="output format")
    PARSER.add_argument('--version', action='version', version = VERSION)
    PARSER.add_argument('--filter', dest='myfilter',nargs='?', default=None, help="comma separated field list")
    PARSER.add_argument("verb", metavar="verb" , help="verb")
    PARSER.add_argument("identifier", metavar="identifier" , nargs='?', help="command")
    ARGS = PARSER.parse_args()
    import sys
    DEBUGLEVEL = ARGS.debuglevel or logging.WARNING
    logging.basicConfig(level = DEBUGLEVEL)
    if ARGS.verb == "i":
        info(ARGS.identifier, ARGS. outputformat)
    elif ARGS.verb in ("s","l"):
        if ARGS.verb == "s":
            query = ARGS.identifier
        else:
            query = ""
        from pydsl.Memory.Search.Searcher import MemorySearcher
        from pydsl.Memory.Search.Indexer import Indexer
        searcher = MemorySearcher([Indexer(x) for x in GLOBALCONFIG.memorylist])
        myfilter = None
        if ARGS.myfilter:
            myfilter = ARGS.myfilter.split(',')
        if ARGS.outputformat == "str":
            print(search_pp(searcher.search(ARGS.identifier), myfilter))
        elif ARGS.outputformat == "json":
            import json
            print(json.dumps(list(filterset(searcher.search(ARGS.identifier), myfilter))))
    else:
        print("Unknown verb")
        print(TUSAGE)
        sys.exit(-1)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""Abstract Classes"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
LOG = logging.getLogger(__name__)

class Singleton(type):
    """singleton pattern metaclass"""
    #Only problem here is that classes can't have two metaclasses
    def __init__(cls, name, bases, dct):
        cls.__instance = None
        type.__init__(cls, name, bases, dct)

    def __call__(cls, *args, **kw):
        if cls.__instance is None:
            cls.__instance = type.__call__(cls, *args, **kw)
        return cls.__instance


class InmutableDict(dict):
    """A dict with a hash method for dictionary use"""
    def __hash__(self):
        if not self:
            return 0
        items = tuple(self.items())
        res = hash(items[0])
        for item in items[1:]:
            res ^= hash(item)
        return res

    def __eq__(self, other):
        if len(self.keys()) != len(other.keys()):
            return False
        for key in self:
            if other[key] != self.__getitem__(key):
                return False
        return True


class Indexable(object):
    """ This class is searchable """
    def summary(self): #-> InmutableDict:
        raise NotImplementedError

    @classmethod
    def ancestors(cls):
        result = [cls.__name__]
        for x in cls.__bases__:
            if issubclass(x, Indexable):
                if not x in result:
                    result += x.ancestors()
        return tuple(result)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""LR0 unfinished implementation"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
LOG = logging.getLogger(__name__)
from pydsl.Grammar.Parser.Parser import BottomUpParser
from pydsl.Grammar.Symbol import NonTerminalSymbol, TerminalSymbol, EndSymbol, Symbol
from pydsl.Grammar.BNF import NonTerminalProduction

def __check_action(action):
    """Valid actions string"""
    return action in ["Accept", "Shift", "Reduce", "Fail"]

def __build_item_closure(itemset, productionruleset):
    """Build input itemset closure """
    #For every item inside current itemset, if we have the following rule:
    #  xxx <cursor><nonterminalSymbol> xxx  append every rule from self._productionruleset that begins with that NonTerminalSymbol
    LOG.debug("__build_item_closure: Begin")
    if not isinstance(itemset, LR0ItemSet):
        raise TypeError
    import copy
    resultset = copy.copy(itemset)
    changed = True
    while changed:
        changed = False
        for currentitem in resultset.itemlist:
            nextsymbol = currentitem.next_symbol()
            if nextsymbol is None:
                break
            for rule in productionruleset.productionrulelist:
                if rule.leftside[0] == nextsymbol:
                    newitem = LR0Item(rule, 0)
                    if newitem not in resultset.itemlist:
                        resultset.append_item(newitem)
                        changed = True
                if rule.rightside[0] == nextsymbol:
                    newitem = LR0Item(rule, 0)
                    if newitem not in resultset.itemlist:
                        resultset.append_item(newitem)
                        changed = True
    return resultset

def item_set_goto(itemset, inputsymbol, productionruleset):
    """returns an itemset
    locate inside itemset every element with inputsymbol following cursor
    for every located item, append its itemclosure"""
    resultset = LR0ItemSet()
    for item in itemset.itemlist:
        if item.next_symbol() == inputsymbol:
            newitem = LR0Item(item.rule, item.position + 1)
            resultset.append_item(newitem)
    return __build_item_closure(resultset, productionruleset)

def build_states_sets(productionruleset):
    """Build states sets"""
    LOG.debug("buildStatesSets: Begin")
    result = []
    symbollist = productionruleset.getSymbols()
    #mainproductionrule = productionruleset.getMainRule()
    mainproductionrule = NonTerminalProduction("Extended initial", [NonTerminalSymbol("EI")] , [productionruleset.getInitialSymbol()])
    mainproductionruleitem = LR0Item(mainproductionrule, 0)
    mainproductionruleitemset = LR0ItemSet()
    mainproductionruleitemset.append_item(mainproductionruleitem)
    index0 = __build_item_closure(mainproductionruleitemset, productionruleset)
    LOG.debug("buildStatesSets: mainsymbol closure: " + str(index0))
    result.append(index0)
    changed = True
    #returns a set of itemsets
    while changed:
        changed = False
        for itemset in result:
            for symbol in symbollist:
                if symbol in itemset:
                    break
                newitemset = item_set_goto(itemset, symbol, productionruleset)
                found = False
                for curreintitemset in result:
                    if curreintitemset == newitemset:
                        found = True
                        changed = True
                        itemset.append_transition(symbol, curreintitemset)
                        break
                if not found and (newitemset not in result):
                    changed = True
                    result.append(newitemset)
                    itemset.append_transition(symbol, newitemset)
    return result

def __slr_build_parser_table(productionruleset):
    """SLR method to build parser table"""
    LOG.debug("__slr_build_parser_table: Begin")
    result = ParserTable()
    #when buildStatesSets finishes its job
    statesset = build_states_sets(productionruleset)
    for itemindex in range(len(statesset)):
        itemset = statesset[itemindex]
        LOG.debug("__slr_build_parser_table: Evaluating itemset:" + str(itemset))
        for symbol in productionruleset.getTerminalSymbols() + [EndSymbol]:
            numberoptions = 0
            for lritem in itemset.itemlist:
                #if cursor is before a terminal, and there is a transition to another itemset with the following terminal, append shift rule
                if isinstance(symbol, TerminalSymbol) and lritem.next_symbol() == symbol and symbol in itemset:
                    destinationstate = statesset.index(itemset.get_transition(symbol))
                    result.append_rule(itemindex, symbol, "Shift", destinationstate)
                    numberoptions += 1
                #if cursor is at the end of the rule, then append reduce rule and go transition
                if lritem.previous_symbol() == symbol and lritem.is_last_position() and symbol != productionruleset.getInitialSymbol():
                    result.append_reduction(itemindex, symbol, itemindex, lritem)
                    numberoptions += 1
                #if cursor is at the end of main rule, and current symbol is end, then append accept rule
                if isinstance(symbol, EndSymbol) and lritem.is_last_position() and symbol == productionruleset.getInitialSymbol():
                    result.append_rule(itemindex, symbol, "Accept", itemindex)
                    numberoptions += 1
            if numberoptions == 0:
                LOG.debug("No rule found to generate a new parsertable entry ")
                LOG.debug("symbol: " + str(symbol))
                LOG.debug("itemset: " + str(itemset))
            if numberoptions > 1:
                from pydsl.Exceptions import LRConflictException
                raise LRConflictException
    return result
    
class ParserTable(object):
    """ Stores a state/symbol/action/new state relation """
    def __init__(self):
        self.__internalstate = None
        self.set_initial_state()
        #Default for every state: Fail state
        self.__table = {}

    def __str__(self):
        return "<ParserTable - State:" + str(self.__internalstate) + " Table: " + str(self.__table) + ">"

    def append_rule(self, state, symbol, action, destinationstate):
        """Appends a new rule"""
        if not __check_action(action) or action == "Reduce":
            raise TypeError
        if not state in self.__table:
            self.__table[state] = {}
        self.__table[state][symbol.name] = {"action":action, "dest":destinationstate}

    def append_reduction(self, state, symbol, destinationstate, rule):
        """Appends a reduce action. It also stores which ProductionRule must follow on reduction"""
        if not state in self.__table:
            self.__table[state] = {}
        self.__table[state][symbol.name] = {"action":"Reduce", "dest":destinationstate, "rule":rule}

    def current_state(self):
        """returns current state"""
        return self.__internalstate

    def insert_symbol(self, symbol):
        """change internal state, return action"""
        if not isinstance(symbol, Symbol):
            LOG.error("Bad arg for insert_symbol function")
            raise TypeError
        if self.__internalstate not in self.__table:
            raise IndexError
        if symbol not in self.__table[self.__internalstate]:
            self.set_initial_state()
            return {"action":"Fail"}
        result = self.__table[self.__internalstate, symbol]
        newstate = result["dest"]
        self.__internalstate = newstate
        return result
        
    def set_initial_state(self):
        """sets initial state to 0"""
        self.__internalstate = 0


class LR0Item(object):
    """LR0 table item"""
    def __init__(self, rule, cursorposition):
        if not isinstance(rule, NonTerminalProduction):
            raise TypeError
        self.rule = rule
        self.position = cursorposition
        #TODO: Verificar que la posicion es menor o igual que la longitud de la parte derecha de la regla

    def __str__(self):
        return "<LR0Item - Rule: " + str(self.rule) + " Position: " + str(self.position) + ">"

    def __eq__(self, other):
        if not isinstance(other, LR0Item):
            raise TypeError
        if self.position != other.position:
            return False
        if self.rule != other.rule:
            return False
        return True

    def previous_symbol(self):
        """returns cursor's previous symbol"""
        if self.position == 0:
            return None
        return self.rule.rightside[self.position-1]

    def next_symbol(self):
        """returns the symbol located after cursor"""
        try:
            return self.rule.rightside[self.position]
        except IndexError:
            return None

    def is_last_position(self):
        """Returns true if cursor if after last element"""
        return self.position > len(self.rule.rightside)

class LR0ItemSet(object):
    """Stores LR0Items, and a dic with symbols and destination states"""
    def __init__(self):
        self.itemlist = []
        self.__transitiondic = {}

    def __str__(self):
        result = "<LR0ItemSet: \n"
        for item in self.itemlist:
            result += str(item) + ","
        result += "transitions:" + str(self.__transitiondic)
        result += ">"
        return result

    def __eq__(self, anotherset):
        """Tests on itemlist equality"""
        if not isinstance(anotherset, LR0ItemSet):
            raise TypeError
        if len(self.itemlist) != len(anotherset.itemlist):
            return False
        for element in self.itemlist:
            if element not in anotherset.itemlist:
                return False
        return True

    def append_item(self, item):
        """Apend new item to set"""
        if not isinstance(item, LR0Item):
            raise TypeError
        self.itemlist.append(item)

    def append_transition(self, symbol, targetset):
        """Appends a transition"""
        if symbol.name in self.__transitiondic:
            return
        self.__transitiondic[symbol.name] = targetset

    def __contains__(self, symbol):
        return symbol.name in self.__transitiondic

    def get_transition(self, symbol):
        """gets a transition"""
        return self.__transitiondic[symbol.name]

class LR0Parser(BottomUpParser):
    """LR0 bottomup parser. Not finished"""
    def __init__(self, productionruleset):
        #TODO: Build extended productionruleset before calling parent constructor
        BottomUpParser.__init__(self, productionruleset)
        self.__stack = [] 
        #Add main item to itemsclosure with cursor at 0 position
        self.__parsertable = __slr_build_parser_table(productionruleset)
        #build GoTo and Action Table from ProductionRuleSet

    def check_word(self, tokenlist):
        """see parent docstring"""
        LOG.debug("check_word: Begin")
        self.__parsertable.set_initial_state()
        #empty stack

        #convert tokens to terminalsymbols
        #iterate over symbollist
        LOG.debug("check_word: checking list: " + str(tokenlist))
        for token in tokenlist:
            newdic = self.__parsertable.insert_symbol(token)
            currentstate = self.__parsertable.current_state()
            action = newdic["action"]
            if action == "Reduce":
                reductionrule = newdic["rule"]
                #TODO extract len(right side) of the rule and insert left side
                for unused in range(len(reductionrule.rightside)):
                    self.__stack.pop()
                self.__stack.append((currentstate, reductionrule.leftside))
            if action == "Shift":
                self.__stack.append((currentstate, token))
            if action == "Fail":
                return False
            if action == "Accept":
                return True
        return False


    def get_symbol_token_map(self, tokenlist):
        """ returns a SymbolTokenMap with best guesses """
        #Symbol to token map can be retrieved from parsertable. It must store left side of rule on reduction 
        #TODO : think about this
        pass




def function(inputdic, inputgt, outputgt):
    if inputdic["input"].string == "": 
        return {}
    numberstr = inputdic["input"].string
    decimalnumber = int(numberstr, 2)
    decimalstr = str(decimalnumber)
    return {"output":decimalstr}


iclass = "PythonTransformer"
inputdic = {"input":"binary"}
outputdic = {"output":"integer"}



#!/usr/bin/python
# -*- coding: utf-8 -*-

def function(inputdic, inputgt, outputgt):
    strfloat =  inputdic['input']
    realfloat = float(strfloat)
    realint = int(realfloat)
    return {"output":str(realint)}


inputdic = { "input":"float"}
outputdic = {"output":"integer"}
iclass = "PythonTransformer"



import pika
import json
import threading
import time

credentials = pika.PlainCredentials('root', '1234')

class Agent(threading.Thread):
    def __init__(self, exchange, name, actiondictionary):
        """Subscription is the exchange key to connect to aside the name"""
        threading.Thread.__init__(self)
        self.daemon = True
        self.exchange = exchange
        if name == "output":
            raise Exception("output is a protected name")
        self.name = name 
        self.connection = pika.BlockingConnection(pika.ConnectionParameters( host='localhost', credentials=credentials))
        self.channeloutput = self.connection.channel()
        self.channelinput = self.connection.channel()
        result = self.channelinput.queue_declare(exclusive=True)
        self.queue_name = result.method.queue
        self.actiondictionary = actiondictionary
        routing_key = "std." + self.name
        self.channelinput.queue_bind(exchange=self.exchange,
                           queue=self.queue_name,
                           routing_key=routing_key)
        

    def __del__(self):
        self.connection.close()
        
    def emit_message(self, destination, message):
        if destination == 'output':
            routing_key='out.*'
        else:
            routing_key =  'std.' + destination
        obj = {'message':message, 'destination':destination, 'source':self.name}
        self.channeloutput.basic_publish(exchange=self.exchange, routing_key=routing_key, body=json.dumps(obj))

    def onmessage(self, ch, method, properties, body):
        obj = json.loads(body)
        #default signature for function : (parentobj, messagedictionary)
        #special inputs: input
        #special output: output
        return self.actiondictionary[obj['source']](self, obj)

    def run(self):
        self.channelinput.basic_consume(self.onmessage,
                              queue=self.queue_name,
                              no_ack=False)
        self.channelinput.start_consuming()

class AgentNetwork(object):
    #generates the network from a list
    def __init__(self, exchange, initlist):
        """initlist allows agent declaration,
        outputsubscription tells the key to connect to for outputs"""
        self.exchange = exchange
        self.connection = pika.BlockingConnection(pika.ConnectionParameters( host='localhost', credentials=credentials))
        self.channelinput = self.connection.channel()
        self.channelinput.exchange_declare(exchange=self.exchange, type='topic')
        result = self.channelinput.queue_declare(exclusive=True)
        self.queue_name = result.method.queue
        self.channeloutput = self.connection.channel()
        #declare exchange
        self.agentlist = []
        for (name,dic) in initlist:
            self.agentlist.append(Agent(self.exchange, name, dic))
        #call subprocess per agent
        routing_key = 'out.*'
        self.channelinput.queue_bind(exchange=self.exchange,
                           queue=self.queue_name,
                           routing_key=routing_key)

    def __del__(self):
        self.connection.close()
        
    def send_input(self, message):
        """Sends an input to the network"""
        obj = {'message':message, 'destination':'*', 'source':'input'}
        for x in self.agentlist:
            routing_key='std.'+x.name
            self.channeloutput.basic_publish(exchange=self.exchange, routing_key=routing_key, body=json.dumps(obj))


    def call(self,message):
        slept = 0
        for element in self.agentlist:
            element.start()
        self.send_input(message)
        time.sleep(0.5)
        while slept < 9:
            method_frame, header_frame, body = self.channelinput.basic_get(queue = self.queue_name)
            if method_frame:
                self.channelinput.basic_ack(delivery_tag=method_frame.delivery_tag)
                return json.loads(body)
            time.sleep(0.5)
            slept += 0.5
        return None




#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"


class GrammarDefinition(object):
    def __init__(self):
        pass

    def enum(self):
        raise NotImplementedError

    @property
    def first(self):# -> set:
        raise NotImplementedError

    @property
    def minsize(self):# -> int:
        return 0

    @property
    def maxsize(self):
        return None

    def alphabet(self):
        """Returns the alphabet required by this grammar"""
        raise NotImplementedError

class PLYGrammar(GrammarDefinition):
    """PLY based grammar"""
    def __init__(self, module):
        GrammarDefinition.__init__(self)
        self.module = module

    @property
    def maxsize(self):
        raise NotImplementedError

    @property
    def minsize(self):
        raise NotImplementedError

class RegularExpressionDefinition(GrammarDefinition):
    def __init__(self, regexp, flags = 0):
        if not isinstance(regexp, str):
            raise TypeError
        GrammarDefinition.__init__(self)
        self.regexpstr = regexp
        self.flags = flags
        import re
        self.regexp = re.compile(regexp, flags)

    @property
    def first(self):# -> set:
        i = 0
        while True:
            if self.regexpstr[i] == "^":
                i+=1
                continue
            if self.regexpstr[i] == "[":
                return [x for x in self.regexpstr[i+1:self.regexpstr.find("]")]]
            return self.regexpstr[i] 

    def __getattr__(self, attr):
        return getattr(self.regexp, attr)

class JsonSchema(GrammarDefinition, dict):
    pass

class MongoGrammar(GrammarDefinition, dict):
    @property
    def first(self):
        return "{"



#!/usr/bin/python3

# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""
extracts input slices that are a Type 
"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Néstor Arocha"
__email__ = "nesaro@gmail.com"

import logging
from pydsl.Exceptions import BadFileFormat
from pydsl.Interaction.Shell import parse_shell_dict, open_files_dict 

def checkfun(inputdic, auxboarddic, inputgt, outputgt):
    output = auxboarddic["checker"]({"string":inputdic["input"], "grammar":CURRENTGRAMMAR})
    if not output:
        return output
    return {"output":str(output["output"])}

def extract(expression = None, outputfiledic = None, pipemode = None, inputfiledic = None, **kwargs):
    #Generating and connecting output
    #listen to user, open read file, or other
    #configure output, write file, or other
    from pydsl.Function.Python import HostPythonTransformer
    maingt = HostPythonTransformer({"input":"cstring"},{"output":"TrueFalse"},{"checker":"GrammarChecker"}, checkfun) 
    if expression and outputfiledic: #input type: expression #output: file
        myexpression = {"input":expression}
        outputdic = parse_shell_dict(outputfiledic)
        resultdic = maingt(myexpression, outputdic)
        from pydsl.Interaction.Shell import save_result_to_output
        save_result_to_output(resultdic, outputdic)
        return resultdic
    elif expression and not outputfiledic:
        result = _slice(maingt,expression)
        print(result)
        return result #FIXME: Solo en el modo expresion se espera de resultado para test 
    elif inputfiledic:
        inputdic = parse_shell_dict(inputfiledic)
        outputdic = {"output":"stdout"}
        if outputfiledic:
            outputdic = parse_shell_dict(outputfiledic)
        stringdic = open_files_dict(inputdic)
        resultdic = maingt(stringdic)
        resultdic = bool_dict_values(resultdic)
        from pydsl.Interaction.Shell import save_result_to_output
        save_result_to_output(resultdic, outputdic)
    elif pipemode:
        from pydsl.Interaction.Shell import StreamFileToTransformerInteraction
        assert(len(maingt.inputchanneldic) == 1)
        assert(len(maingt.outputchanneldic) == 1)
        inputname = list(maingt.inputchanneldic.keys())[0]
        outputname = list(maingt.outputchanneldic.keys())[0]
        interactor = StreamFileToTransformerInteraction(maingt, {inputname:"stdin"} , {outputname:"stdout"})
        interactor.start()
    else:
        raise Exception
    return True

def _slice(maingt, inputdata):
    totallen = len(inputdata)
    from pydsl.Memory.Loader import load_grammar_tool
    currenttype = load_grammar_tool(CURRENTGRAMMAR)
    try:
        maxl = currenttype.maxsize
    except NotImplementedError:
        maxl = totallen
    try:
        minl = currenttype.minsize
    except NotImplementedError:
        minl = 1
    maxwsize = maxl - minl + 1
    result = []
    for i in range(totallen):
        for j in range(i+minl, min(i+maxwsize+1, totallen+1)):
            check = maingt({"input":inputdata[i:j]})["output"]
            if check == "True":
                result.append((i,j, inputdata[i:j]))
    return result

            #TODO check alphabet

if __name__ == "__main__":
    import argparse
    TUSAGE = "usage: %(prog)s [options] type"
    PARSER = argparse.ArgumentParser(usage = TUSAGE)
    PARSER.add_argument("-d", "--debuglevel", action="store", type=int, dest="debuglevel", help="Sets debug level")
    PARSER.add_argument("-i", "--inputfile", action="store", dest="inputfiledic", help="input filename dict")
    PARSER.add_argument("-o", "--outputfile", action="store", dest="outputfiledic", help="output filename dict")
    PARSER.add_argument("-e", "--expression", action="store", dest="expression", help="input expression")
    PARSER.add_argument("tname", metavar="tname", help="Type name")
    ARGS = PARSER.parse_args()
    import sys
    if (ARGS.outputfiledic and not ARGS.expression) and (ARGS.outputfiledic and not ARGS.inputfiledic):
        PARSER.error("options -o require -e or -i")
    DEBUGLEVEL = ARGS.debuglevel or logging.WARNING
    
    logging.basicConfig(level = DEBUGLEVEL)
    try: 
        global CURRENTGRAMMAR
        CURRENTGRAMMAR = ARGS.tname
    except BadFileFormat:
        print("Error reading input file")
        sys.exit(1)
    except KeyError as le:
        print("Unable to load " + str(le))
        sys.exit(1)
    try:
        result = extract(**vars(ARGS))
    except EOFError:
        sys.exit(0)
    if not result:
        sys.exit(-1)
    sys.exit(0)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""Wrapper Classes"""

#FIXME: python3 support only

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
LOG = logging.getLogger(__name__)
from pydsl.Memory.Loader import load

class Content:
    """String wrapper"""
    def __init__(self, content):
        self.content = content
        self.grammar = None
            

    def check(self, grammar):
        return grammar in self.guess()

    def available_alphabets(self):
        return []

    def available_grammars(self):
        from pydsl.Guess import Guesser
        guess = Guesser()
        return guess(self.content)

    def select_grammar(self, grammar):
        if grammar not in self.available_grammars():
            print(self.available_grammars())
            raise Exception
        self.grammar = grammar

class FunctionsMeta(type):
    def __dir__(cls):
        from pydsl.Memory.Search.Searcher import MemorySearcher
        from pydsl.Memory.Search.Indexer import Indexer
        from pydsl.Config import GLOBALCONFIG
        searcher = MemorySearcher([Indexer(x) for x in GLOBALCONFIG.memorylist])
        return [ x["identifier"] for x in searcher.search({"ancestors":{"$in":"Function"}})]

    def __getattr__(cls, key):
        return load(key)

    @staticmethod
    def available_transforms(content):
        from pydsl.Memory.Search.Searcher import MemorySearcher
        from pydsl.Memory.Search.Indexer import Indexer
        from pydsl.Config import GLOBALCONFIG
        searcher = MemorySearcher([Indexer(x) for x in GLOBALCONFIG.memorylist])
        resultlist = []
        if content.grammar:
            grammarlist = [content.grammar]
        else:
            grammarlist = content.available_grammars()
        for element in grammarlist:
            resultlist += searcher.search({"input":{"$part":{"input":element}}})
        return [x['identifier'] for x in resultlist]

class FunctionPool(metaclass=FunctionsMeta):
    pass

class NetworkPool:
    pass




#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"


class AlphabetDefinition(object):
    """Defines an alphabet"""
    @property
    def symbols(self):
        """Returns a list of allowed symbols"""
        raise NotImplementedError

class AlphabetDictDefinition(AlphabetDefinition):
    """Uses a dict of grammardefinitions"""
    def __init__(self, grammardict):
        from pydsl.Memory.Loader import load
        self.grammardict = {}
        for x in grammardict:
            self.grammardict[x] = load(grammardict[x])

    @property
    def symbols(self):
        return list(self.grammardict.keys())

class Encoding(AlphabetDefinition):
    """Defines an alphabet using an encoding string"""
    def __init__(self, encoding):
        self.encoding = encoding


#!/usr/bin/python
# -*- coding: utf-8 -*-

"""Python  hierarchy class to dot"""

def function(inputdic, inputgt, outputgt):
    if inputdic["input"].string == "": 
        return {}
    filename = inputdic["input"]
    knownclass = {} #{"Name":[childlist]}
    import re
    with open(filename, encoding='utf-8') as f:
        for line in f:
            match = re.match("^class (\w+)(\(.+\))?:", line)
            if match:
                if match.lastindex == 1:
                    knownclass[match.group(1)] = None
                else:
                    knownclass[match.group(1)] = match.group(2)[1:-1].split(",")
    #print(knownclass)
    output = "digraph graphname \n{ "
    for key in knownclass:
        if knownclass[key] is None:
            output += '"' + key + '"\n'
        else:
            for child in knownclass[key]:
                output += '"' + key + '" -> "' + child + "\"\n"
    output += "}"
    return {"output":output}


iclass = "PythonTransformer"
inputdic = {"input":"unixPathGrammar"}
outputdic = {"output":"cstring"}




#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (c) 2008-2013 Nestor Arocha Rodriguez

"""LL(1) Recursive Descent Lexer
second recipe of the book "Language implementation patterns

grammar NestedNameList;
list : '[' elements ']' ; // match bracketed list
elements : element (',' element)* ; // match comma-separated list
element : NAME | list ; // element is name or nested list
NAME : ('a'..'z' |'A'..'Z' )+ ; // NAME is sequence of >=1 letter
"""

tokenlist = ["NAME", "COMMA", "LBRACK", "RBRACK","EOF_TYPE"]



def function(inputdic, inputgrammar, outputdic):
    def nextToken(tl):
        import re
        while tl:
            current = tl[0]
            if current == "/":
                comment(tl)
                continue
            elif current == " ":
                tl.pop(0)
                continue
            elif current == ",":
                tl.pop(0)
                return "COMMA", current
            elif current == "[":
                tl.pop(0)
                return "LBRACK", current
            elif current == "]":
                tl.pop(0)
                return "RBRACK", current
            elif re.match("[a-zA-Z]", current):
                return name(tl)
            else:
                raise Exception
        return "EOF_TYPE", ""


    def name(tl):
        import re
        string = ""
        while tl and re.match("[a-zA-Z]", tl[0]):
            char = tl.pop(0)
            string += char
        return "NAME", string

    inputlist = [x for x in inputdic["input"]]
    result = [nextToken(inputlist)]
    while result[-1][0] != "EOF_TYPE":
        result.append(nextToken(inputlist))
    return {"output":result}


iclass = "PythonTransformer"
inputdic = {"input":"cstring"}
outputdic = {"output":"cstring"}





#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""Global (per execution) elements"""
from pydsl.Memory.List import EncodingStorage

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

from pydsl.Abstract import Singleton
import logging
LOG = logging.getLogger(__name__)
from pkg_resources import resource_filename, DistributionNotFound


def generate_memory_list(): #-> list:
    """loads default memories"""
    result = []
    from pydsl.Memory.Directory import DirStorage
    from pydsl.Memory.Dict import RegexpDictStorage
    try:
        dirname = resource_filename("pydsl.contrib", "")
    except DistributionNotFound:
        pass
    else:
        result.append(DirStorage(dirname + "/grammar/"))
        result.append(DirStorage(dirname + "/board/"))
        result.append(DirStorage(dirname + "/transformer/"))
        result.append(RegexpDictStorage(dirname + "/dict/regexp.dict"))
        result.append(EncodingStorage(dirname + "/list/encoding.py"))
    return result


class GlobalConfig(object):
    """Execution time global configuration"""
    def __init__(self, persistent_dir=None, debuglevel=40):
        self.persistent_dir = persistent_dir
        self.__memorylist = None  # default memories, sorted by preference
        self.__debuglevel = debuglevel
        self.lang = "es"
        if self.persistent_dir is None:
            try:
                import os
                if not os.path.exists(os.environ['HOME'] + "/.pydsl/"):
                    os.mkdir(os.environ['HOME'] + "/.pydsl/")
                self.persistent_dir = os.environ['HOME'] + "/.pydsl/persistent/"
                if not os.path.exists(self.persistent_dir):
                    os.mkdir(self.persistent_dir)
            except (OSError, KeyError):
                LOG.exception("Unable to create persistent dir")

    def load(self, filename):
        """Load config from file"""
        raise NotImplementedError

    def save(self):
        """Save config to file"""
        raise NotImplementedError

    @property
    def memorylist(self):
        if self.__memorylist is None:
            self.__memorylist = generate_memory_list()
        return self.__memorylist

    @property
    def debuglevel(self):
        return self.__debuglevel

    @debuglevel.setter
    def debuglevel(self, level):
        self.__debuglevel = level

GlobalConfig2 = Singleton('GlobalConfig2', (GlobalConfig, ), {})
GLOBALCONFIG = GlobalConfig2()  # The only instance available


def all_classes(module):# -> set:
    """Returns all classes (introspection)"""
    import inspect
    result = set()
    for name, obj in inspect.getmembers(module):
        if inspect.isclass(obj):
            result.add(obj)
        elif inspect.ismodule(obj):
            if obj.__name__[:6] == "pydsl":
                result = result.union(all_classes(obj))
    return result


def all_indexable_classes(module):# -> set:
    """Returns all indexable classes (introspection)"""
    import inspect
    result = set()
    for name, obj in inspect.getmembers(module):
        from pydsl.Abstract import Indexable
        if inspect.isclass(obj) and issubclass(obj, Indexable):
            result.add(obj)
        elif inspect.ismodule(obj):
            if obj.__name__[:6] == "pydsl":
                result = result.union(all_indexable_classes(obj))
    return result



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""External program Transformer"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"


import logging
import subprocess
from pydsl.Exceptions import ProcessingError
from .Channel import HostChannel
LOG = logging.getLogger("Function.ExternalProgram")

class ExternalProgramFunction(HostChannel):
    """returns stdout from program call"""
    def __init__(self, inputdic, outputdic, programcall):
        #programlist ["program","-s","#{inputchannelname1}","-e","#{inputchannelname2}"]
        if len(outputdic) != 1:
            raise ValueError
        HostChannel.__init__(self, inputdic, outputdic)
        self.__programlist = programcall

    def __call__(self, wdic):
        for inputkey in self.inputchanneldic.keys():
            if inputkey not in wdic:
                LOG.error("Key not found in inputdic")
                raise ProcessingError("Transformer", self)
        result = self.__processWords(wdic)
        if not result:
            raise ProcessingError("Transformer")
        return result

    def __processWords(self, inputdict):
        #inputdict = {channelname:data,}
        if not inputdict:            
            raise Exception
        calllist  = list(self.__programlist)
        for channel, data in inputdict.items():
            channelfound = False
            for index in range(len(calllist)):
                element = calllist[index]
                if element.find("#{"+channel+"}") != -1:
                    channelfound = True
                    calllist[index] = element.replace("#{"+channel+"}", str(data))
            if not channelfound:
                raise Exception("No channel found")
        proc = subprocess.Popen(calllist, stdout=subprocess.PIPE)
        lines = proc.stdout.readlines()
        proc.stdout.close()
        if len(lines) > 1:
            LOG.warning("program execution returned > 1 lines")
        lines = map(lambda x: x.decode('utf-8'), lines)
        lines = "\n".join(lines)
        for key in self.outputchanneldic.keys():
            return {key:lines} #FIXME: only one channel!?
        #returns dict {outputchannelname:informationBlock,}

    @property
    def summary(self):
        inputdic = [ x.identifier for x in self.inputchanneldic.values() ]
        outputdic = [ x.identifier for x in self.outputchanneldic.values() ]
        return {"iclass":"ExternalProgramFunction", "input":inputdic, "output":outputdic }



#!/usr/bin/python
# -*- coding: utf-8 -*-

def function(inputdic, inputgt, outputgt):
    grammarname =  str(inputdic['grammar'])
    from pydsl.Memory.Loader import load_checker
    grammar = load_checker(grammarname)
    result = grammar.check(inputdic['string'])
    return {"output":str(result)}


inputdic = {"grammar":"cstring", "string":"cstring"}
outputdic = {"output":"TrueFalse"}
iclass = "PythonTransformer"



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.


"""Exceptions definitions"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"


import logging
LOG = logging.getLogger(__name__)
ERRORLIST = ["Grammar", "Timeout", "Transformer"]

class ProcessingError(Exception):
    def __init__(self, errortype, source=None):
        Exception.__init__(self)
        if not source:
            source = []
        self.source = source
        assert(errortype in ERRORLIST)
        self.errortype = errortype

    def append(self, element):
        self.source.append(element)

class LRConflictException(Exception):
    pass

class BadFileFormat(Exception):
    def __init__(self, filename):
        Exception.__init__(self)
        self.filename = filename

class ParseError(Exception):
    def __init__(self, msg, offset):
        self.msg = msg
        self.offset = offset
    def __repr__(self):
        return "ParseError(%r, %r)" % (self.msg, self.offset)
    def __str__(self):
        return "%s at position %s" % (self.msg, self.offset + 1)




#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (c) 2008-2013 Nestor Arocha Rodriguez

"""LL(1) Recursive Descent Lexer
second recipe of the book "Language implementation patterns

grammar NestedNameList;
list : '[' elements ']' ; // match bracketed list
elements : element (',' element)* ; // match comma-separated list
element : NAME | list ; // element is name or nested list
NAME : ('a'..'z' |'A'..'Z' )+ ; // NAME is sequence of >=1 letter
"""

#tokenlist = ["NAME", "COMMA", "LBRACK", "RBRACK","EOF_TYPE"]

from pydsl.Alphabet.Lexer import Lexer as _Lexer

class _ListLexer(_Lexer):
    def nextToken(self):
        import re
        from pydsl.Alphabet.Lexer import finalchar
        while self.current != finalchar:
            if self.current == "/":
                self.comment(tl)
                continue
            elif self.current == " ":
                self.consume()
                continue
            elif self.current == ",":
                return self.comma()
            elif self.current == "[":
                return self.lbrack()
            elif self.current == "]":
                return self.rbrack()
            elif re.match("[a-zA-Z]", self.current):
                return self.name()
            else:
                raise Exception
        return "EOF_TYPE", ""

    def comma(self):
        current = self.current
        self.match(",")
        return "COMMA", current

    def lbrack(self):
        current = self.current
        self.match("[")
        return "LBRACK", current

    def rbrack(self):
        current = self.current
        self.match("]")
        return "RBRACK", current

    def name(self):
        import re
        string = ""
        from pydsl.Alphabet.Lexer import finalchar
        while self.current != finalchar and re.match("[a-zA-Z]", self.current):
            string += self.current
            self.consume()
        return "NAME", string


def function(inputdic, inputgrammar, outputdic):
    a = _ListLexer(inputdic["input"])
    result = [a.nextToken()]
    while result[-1][0] != "EOF_TYPE":
        result.append(a.nextToken())
    return {"output":result}


iclass = "PythonTransformer"
inputdic = {"input":"cstring"}
outputdic = {"output":"cstring"}





def function(inputdic, inputgt, outputgt):
    if inputdic["input"].string == "": 
        return {}
    numberstr = inputdic["input"].string
    number = int(numberstr)
    binstr = str(bin(number))[2:]
    return {"output":binstr}


iclass = "PythonTransformer"
inputdic = {"input":"integer"}
outputdic = {"output":"binary"}
